<?php
/**
 * WEB apmokėjimų klasės failas
 * 
 * @package MBSLib
 * @author Valdas Petrulis <vpe@vero.lt>
 */

require_once('MBSBase.class.php');

/**
 * WEB apmokėjimų klasė
 * 
 * Naudojimo pavyzdžiai:
 * {@example examples/simple/bank_bill.php}
 * {@example examples/simple/bank_bill_goods.php}
 * 
 * @package MBSLib
 * @author Valdas Petrulis <vpe@vero.lt>
 * 
 */
class Bank extends MBSBase  {
    
    /**
     * Paslaugos identifikatorius. Apmokestinimo paslaugos, kuri turi konkrečias kainas
     * 
     * @access protected
     *
     * @var int
     */
    var $service_id = 0;
    
    /**
     * Wap paslaugų apmokėjimų klasės konstruktorius
     *
     * Konstruktoriu nustatome paslaugos identifikatorių $service_id ir kanalo identifikatorių $channle_id.<br/>
     *
     * Jeigu $back_url nenurodyta gražinimui bus naudojami URL nurodyti paslaugos aprašymuose.
     * Papildomai galima nurodyti $back_url, tokiu atvėju vartotojas iš Vero MBS sistemos ( po apmokęstinimo )
     * bus gražinamas nurodytą nuorodą.<br/>
     *
     *  Taip pat nuorodoje galima naudoti žymenis:
     *    # %t arba %{transaction_id}% - pakeistas į naudojamos transakcijos ID
     *    # %s or %{service_id}% - pakeistas į naudojamos paslaugos(service) ID
     *    # %p or %{transaction_price} - pakeistas į paslaugos kainą
     *    # %cur or %{transaction_currency} - pakeistas į paslaugos kainos valiutą
     *    # %o or %{operator_id}%- pakeistas į vartotojo operatoriaus (operator) ID
     *    # %a or %{account_id}%- pakeistas į vartotojo identifikatorių (account) ID
     *    # %n or %{operator_name}%	- pakeistas į vartotojo operatoriaus pavadinimą (pvz.: bite_lt, tele2_lv, ect.)
     *
     * Šie žymenis bus pakeisti atitinkamomis reikšmėmis. Pvz.: $back_url nustatome į
     * <b>http://example.lt?trans=%t&s=%s&service=%{service_id}%&a=te</b>
     *
     * Įvykdytos transakcijos id buvo <b>2456</b> naudotos paslaugos (service_id) <b>1</b> tokiu atveju vartotojas bus gražintas nuoroda:
     *
     * <b>http://example.lt?trans=2456&s=1&service=1&a=te</b>
     *
     * @param int $service_id Wap paslaugos identifikatorius
     * @param string $secret_key (nebūtina) Slaptas žodis, kuris naudojamas saugumo parašui
     * @param string $back_url (nebūtina) URL kuriuo gražinti vartotoja po registracijos
     * 
     * @return MBS
     */
    function Bank($service_id, $secret_key=null, $back_url=null) {
    	if (!empty($service_id)) { 
    		$this->service_id = (int)$service_id; 
    	} else { 
    		$this->setError('Service ID is not set');
    	}
    	if (!empty($back_url)) {
    		$this->setBackUrl($back_url);
    	}
    	if (!empty($secret_key)) {
    		$this->setSecretKey($secret_key);
    	}
    	
    	$this->debug('Bank class created. Params service_id: ' . $this->service_id, DEBUG_LEVEL_HIGH);
    }
    
    /**
     * Vartotojo apmokęstinimas redirect būdu.
     * 
     * Atlikus šį metodą vartotojas bus nukreiptas į Vero sistemą ir programos darbas nutrauktas.<br/>
     * 
     * Parametras $params nusako kokie parametrai bus gražinami atgal
     * (žiūrėti į {@link MBS konstruktorių} apie $back_url.)<br/>
     * 
     * @param array $params (nebūtina) Papildomi parametrai, kurie bus gražinami atgal
     * @param string $ident (nebūtina) Unikalus pirkimo identifikatorius partnerio sistemoje
     * @param integer $price (nebūtina) Apmokestinimo kaina (valiutos 1/100 dalimis - centais). Reikalinga, kai paslauga veikia be prekių sąrašo
     * @param string $title (nebūtina) 
     * 
     * @see Bill
     */
    function billRedirect($params=array(), $ident=null, $price=null, $title=null) {
        $this->debug(__FUNCTION__.' was started', DEBUG_LEVEL_HIGH );

        // Vartotojas siunčiamas į suformuotą URL adresą
        $url = $this->getBillUrl($params, $ident, $price);
        if (!empty($url)) {
	        header('Location: '.$url);
	        exit();
        }
    }
    
    /**
     * Gražinama nuoroda, į kurią vartotojas turi būti nukreipiamas (redirect), kad vartotojas būtų apmokęstintas
     * 
     * Parametrai identiški {@link billRedirect} metodui.
     * 
     * @param array $params (nebūtina) Papildomi parametrai, kurie bus gražinami atgal
     * @param string $ident (nebūtina) Unikalus pirkimo identifikatorius partnerio sistemoje
     * @param integer $price (nebūtina) Apmokestinimo kaina (valiutos 1/100 dalimis - centais). Reikalinga, kai paslauga veikia be prekių sąrašo
     * @param string $title (nebūtina) 
     * 
     * @see billRedirect
     */
    function getBillUrl($params=array(), $ident=null, $price=null, $title=null) {
        $this->debug(__FUNCTION__.' was started', DEBUG_LEVEL_HIGH );

        // Formuojami apmokestinimo parametrai
        $params['service']  = $this->service_id;
        $params['back_url'] = $this->back_url;
        if( is_numeric($price) ) {
        	$params['price'] = $price;
        }
        if( !empty($title) ) {
            $params['title'] = $title;
        }
        $params['time'] = time();
        
        // Pasirašomi duomenys
        $params = $this->signArrayLow($params);
        
        return $this->formatUrl(MBS_BANK_URL, $params);
    }
    
    /**
     * Nustatoma ar vartotojo grąžintas iš MBS po apmokestinimo jei taip, apmokestinimo duomenys užkraunami.
     * 
     * @return array Billing transaction data
     */
    function getBillInformation() {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);

        // Check is identify data passed from GET
        if( !empty($_GET['transaction']) ) {
            // Check for correct service_id
            if ( empty($_GET['service']) || $_GET['service']!=$this->service_id ) {
                $this->setError('Service not set or wrong service_id');
                return false;
            }
        	
            // Checking for correct security signature
            if( MBS_BILL_VERIFY_LEVEL=='NONE' ) {
                $verified = true;
            } else if( MBS_BILL_VERIFY_LEVEL=='LOW' ) {
                $verified = $this->verifyArrayLow($_GET);
            } else {
                $verified = $this->verifyArrayHigh($_GET);
            }
            if( !$verified ) {
                $this->setError('Unauthorise access');
                return false;
            }
            
            // Identified information
            $transaction = array(
                'transaction_id' => $_GET['transaction'],
                'service_id' => $_GET['service'],
                'account_id' => $_GET['account'],
                'ident' => $_GET['ident'],
                'status' => $_GET['status'],
                'state' => $_GET['state'],
                'value' => $_GET['value'],
                'price' => $_GET['price'],
                'currency' => $_GET['currency'],
            );
            $this->debug('Transaction data is '.var_export($transaction, true), DEBUG_LEVEL_NORMAL);
            
            return $transaction;
        
        // No billing information in GET parameters
        } else {
            $this->setError('No billing transaction');
        }
    }
    
    /**
     * Patikrinama ar už tranzakciją yra sumokėta.
     *
     * Užmokėta tranzakcija yra jeigu sutampa paslaugos identifikatorius ir tranzakcijos būsena yra "commit".
     *
     * @return boolean
     *
     * @see getStatusDescription
     */
    function isBilled() {
    	$transaction = $this->getBillInformation();
    	if ($this->isError()) {
    		return false;
    	}
    	
    	if ($transaction['status'] == 'commit') {
    		return true;
    	}
    	return false;
    }
}

?>