<?php
/**
 * Direct apmokėjimų klasės failas
 * 
 * @package MBSLib
 * @author Valdas Petrulis <vpe@vero.lt>
 */

require_once('MBSBase.class.php');

/**
 * Direct apmokėjimų klasė
 * 
 * Naudojimo pavyzdžiai:
 * {@example examples/simple/direct_bill_soap.php}
 * {@example examples/simple/notify_direct.php}
 * 
 * @package MBSLib
 * @author Valdas Petrulis <vpe@vero.lt>
 * 
 */
class Direct extends MBSBase  {
    
    /**
     * Paslaugos identifikatorius. Apmokestinimo paslaugos, kuri turi konkrečias kainas
     * 
     * @access protected
     *
     * @var int
     */
    var $service_id;
    
    /**
     * Saugomas apmokėjimo ataskaitų URL
     *
     * @access protected
     *
     * @var string
     */
    var $url_delivery;
    
    /**
     * Vidiniai transakcijų duomenys
     *
     * @access protected
     *
     * @var array
     */
    var $transactions = array();
    
    /**
     * Direct paslaugų apmokėjimų klasės konstruktorius
     *
     * Konstruktoriuje nustatome paslaugos identifikatorių $service_id.
     * Taip pat, suteikiama galimybė sukonfiguruoti paslaugos apmokėjimo ataskaitų URL ir slaptažodį saugumui.
     *
     * @param int $service_id Direct paslaugos identifikatorius
     * @param string $url_delivery (nebūtina) URL kuriuo siųsti apmokėjimo ataskaitas. Jeigu $url_delivery nenurodyta ataskaitų siuntimui bus naudojamas URL nurodytas paslaugos aprašymuose.
     * @param string $secret_key (nebūtina) Slaptas žodis, kuris naudojamas saugumo parašui
     * 
     * @return Direct
     */
    function Direct($service_id, $url_delivery=null, $secret_key=null) {
    	if (!empty($service_id)) { 
    		$this->service_id = (int)$service_id; 
    	} else { 
    		$this->setError('Service ID is not set');
    	}
    	if (!empty($url_delivery)) {
    		$this->setUrlDelivery($url_delivery);
    	}
    	if (!empty($secret_key)) {
    		$this->setSecretKey($secret_key);
    	}
    	
    	$this->debug('Direct class created. Params service_id: ' . $this->service_id, DEBUG_LEVEL_HIGH);
    }
    
    /**
     * Sukonfiguruojamas apmokėjimo ataskaitų URL.
     *
     * Taip pat nuorodoje $url_delivery galima naudoti žymenis:
     *    # %t arba %{transaction_id}% - pakeistas į naudojamos transakcijos ID
     *    # %s or %{service_id}% - pakeistas į naudojamos paslaugos(service) ID
     *    # %p or %{transaction_price} - pakeistas į paslaugos kainą
     *    # %cur or %{transaction_currency} - pakeistas į paslaugos kainos valiutą
     *    # %o or %{operator_id}%- pakeistas į vartotojo operatoriaus (operator) ID
     *    # %a or %{account_id}%- pakeistas į vartotojo identifikatorių (account) ID
     *    # %n or %{operator_name}% - pakeistas į vartotojo operatoriaus pavadinimą (pvz.: bite_lt, tele2_lv, ect.)
     *
     * Šie žymenys bus pakeisti atitinkamomis reikšmėmis. Pvz.: $url_delivery nustatome į
     * <b>http://example.lt?trans=%t&s=%s&service=%{service_id}%&a=te</b>
     *
     * Įvykdytos transakcijos id buvo <b>2456</b> naudotos paslaugos (service_id) <b>1</b> tokiu atveju vartotojas bus gražintas nuoroda:
     *
     * <b>http://example.lt?trans=2456&s=1&service=1&a=te</b>
     *
     * @param string $url_delivery URL kuriuo siųsti apmokėjimo ataskaitas
     */
    function setUrlDelivery($url_delivery) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);
        
        $this->url_delivery = $url_delivery;
        $this->debug('Delivery URL is set to :'.$this->url_delivery, DEBUG_LEVEL_HIGH);
    }
    
    /**
     * Vykdoma tiesioginė SOAP užklausa, kurios metu įvykdomas vartotojo apmokėjimas ir išsiunčimas nurodytas turininio vienetas (SMS, WapPush).
     * Vartotojo apmokėjimą galima vykdyti, jei prieš tai jis užsiregistravo į paslaugą vienu iš MBS kanalų (siųsdamas SMS, WAP redirect ir pan.).
     * Taip pat yra ribojamas turinio siuntimo dažnumas (pvz., ne daugiau kaip 2 SMS per savaitę, 1 SMS per dieną ir pan.).
     * Jei reikalingas apmokėjimo rezultatas, jis bus perduotas vėliau į apmokėjimo ataskaitų URL.
     * 
     * $ident - parametras naudojamas informatyvumo tikslais, bet yra numatyta, kad ateityje gali būti
     * tikrinama ar tas pats vartotojas už tokį ident jau mokėjo valandos bėgyje.
     * Kad nebūtu vykdomi daukartiniai vartotojo apmokestinimai.
     * 
     * Parametras $language - Nepadavinėkite, jei nežinote vartotojo norimos kalbos. Parametras nurodo kokia kalba vartotojas matys pranešimus MBS sistemoje.
     * Taip pat, šis parametras nurodo kokia kalba siųsti apmokestinimo SMS žinutę.
     *
     * $options – parametrai kurie paduodami į tranzakciją ir vėliau galim juos pasiimti jeigu
     * paslaugos kūrimo metu į nuorodas yra įrašyti forma %{my_<parametro vardas>}%. Galima
     * naudoti parametrą url kuris bus nurodo į kokią nuorodą grąžinti po apmokestinimo detaliau
     * aprašyta prie MBS konstruktoriaus.
     *
     * Norint paduoti toki masyva:
     * <code>
     * array(
     *  'key1'=>'value1',
     *  'key2'=>'value2',
     * )
     * </code>
     * 
     * Reikia jį transformuoti į:
     * <code>
     * array(
     *    array('name'=>'key1', 'value'=>'value1'),
     *    array('name'=>'key2', 'value'=>'value2'),
     * )
     * </code> 
     *
     * @param int $account_id Apmokestinamo vartotojo unikalus ID
     * @param string $ident (nebūtina) Unikalus pirkimo identifikatorius išorinėje sistemoje (tarkim pardavinejat melodijas, tai gali būti melodijos ID ar pavadinimas)
     * @param string $mt_type Vartotojo informavimo tipas (SMS, WapPush)
     * @param string $mt_text Siunčiamo SMS tekstas (jei WapPush, nuorodos antraštė)
     * @param string $mt_url (nebūtina) Jei siuntinio tipas WapPush, siunčiamo WapPush nuoroda 
     * @param integer $price (nebūtina) Apmokestinimo kaina, reikia nurodyti, kai paslauga turi kelias kainas (valiutos 1/100 dalimis - centais)
     * @param string $currency (nebūtina) Kainos valiuta (ISO 4217 standartu, EUR, LTL, LVL ir tt.)
     * @param string $language (nebūtina) Kalbos kodas (ISO 639-1 standartu, EN, RU, LT ir tt.)
     * @param array $options (nebūtina) Papildomi parametrai
     * 
     * @return struct|boolean Sėkmės atvėju gražinama tranzakcijos duomenų struktūra (@see self::getTransaction()), nesėkmės false
     */
    function bill($account_id, $ident=null, $mt_type=null, $mt_text=null, $mt_url=null, $price=null, $currency=null, $language=null, $options=array()) {
    	$this->debug(__FUNCTION__.' was started', DEBUG_LEVEL_HIGH);
    	
    	$options['url_delivery'] = $this->url_delivery;

    	$this->initSoap(MBS_DIRECT_SOAP_URL);
		$transaction = $this->_SOAPCall('MBSBill', array(
            'service_id'=> $this->service_id,
            'account_id'=> $account_id,
            'ident'     => $ident,
            'price'     => $price,
            'currency'  => $currency,
            'mt_type'   => $mt_type,
            'mt_text'   => $mt_text,
            'mt_url'    => $mt_url,
            'language'  => $language, 
            'options'   => $this->prepareParams($options),
		));
		if ($this->isError()) {
            return false;
        }
		
        $this->transactions[$transaction['transaction_id']] = $transaction;
        return $transaction;
    }
    
    /**
     * Vykdoma tiesioginė SOAP užklausa, kurios metu įvykdomas vartotojo apmokėjimas ir išsiunčimas nurodytas turininio vienetas (SMS, WapPush).
     * Vartotojo apmokėjimą galima vykdyti, jei prieš tai jis užsiregistravo į paslaugą vienu iš MBS kanalų (siųsdamas SMS, WAP redirect ir pan.).
     * Taip pat yra ribojamas turinio siuntimo dažnumas (pvz., ne daugiau kaip 2 SMS per savaitę, 1 SMS per dieną ir pan.).
     * Jei reikalingas apmokėjimo rezultatas, jis bus perduotas vėliau į apmokėjimo ataskaitų URL.
     *
     * @param string $account_msisdn Apmokestinamo vartotojo telefono numeris (MSISDN), tarptautiniu formatu „370xxxyyyyy“
     * @param string $operator (nebūtina) Apmokestinamo vartotojo operatorius (bite_lt, tele2_lv ir tt.). Nurodoma, jei partneris tikrai žino apmokestinamo vartotojo operatorių
     * @param string $ident (nebūtina) Unikalus pirkimo identifikatorius išorinėje sistemoje (tarkim pardavinejat melodijas, tai gali būti melodijos ID ar pavadinimas)
     * @param string $mt_type Vartotojo informavimo tipas (SMS, WapPush)
     * @param string $mt_text Siunčiamo SMS tekstas (jei WapPush, nuorodos antraštė)
     * @param string $mt_url (nebūtina) Jei siuntinio tipas WapPush, siunčiamo WapPush nuoroda 
     * @param integer $price (nebūtina) Apmokestinimo kaina, reikia nurodyti, kai paslauga turi kelias kainas (valiutos 1/100 dalimis - centais)
     * @param string $currency (nebūtina) Kainos valiuta (ISO 4217 standartu, EUR, LTL, LVL ir tt.)
     * @param string $language (nebūtina) Kalbos kodas (ISO 639-1 standartu, EN, RU, LT ir tt.)
     * @param array $options (nebūtina) Papildomi parametrai, @see self::bill()
     * 
     * @return struct|boolean Sėkmės atvėju gražinama tranzakcijos duomenų struktūra (@see self::getTransaction()), nesėkmės false
     */
    function billByMSISDN($account_msisdn, $operator=null, $ident=null, $mt_type=null, $mt_text=null, $mt_url=null, $price=null, $currency=null, $language=null, $options=array()) {
    	$this->debug(__FUNCTION__.' was started', DEBUG_LEVEL_HIGH);
        
        $options = (array) $options;
        if( !empty($this->url_delivery) ) {
        	$options['url_delivery'] = $this->url_delivery;
        }

        $this->initSoap(MBS_DIRECT_SOAP_URL);
        $transaction = $this->_SOAPCall('MBSBillByMSISDN', array(
            'service_id'=> $this->service_id,
            'account_msisdn'=> $account_msisdn,
            'operator'  => $operator,
            'ident'     => $ident,
            'price'     => $price,
            'currency'  => $currency,
            'mt_type'   => $mt_type,
            'mt_text'   => $mt_text,
            'mt_url'    => $mt_url,
            'language'  => $language, 
            'options'   => $this->prepareParams($options),
        ));
        if ($this->isError()) {
            return false;
        }
        
        $this->transactions[$transaction['transaction_id']] = $transaction;
        return $transaction;
    }
    
    /**
     * Gražinama norimos tranzakcijos informacija.
     * Jeigu atitinkamos tranzakcijos nėra arba tranzakcijos parametrai - paslaugos identifikatoriai nesutampa su reikalaujamos tranzakcijos, gražinama false reikšmė.
     * 
     * Tranzakcijos duomenų struktūra:
     * <code>
     * array(
     *    'transaction_id' - Unikalus transakcijos ID
     *    'service_id' - Unikalus paslaugos ID 
     *    'ident' - Unikalus pirkimo identifikatorius išorinėje sistemoje 
     *    'operator' - Apmokestinamo vartotojo operatorius (bite_lt, tele2_lv ir tt.)
     *    'account_id' - Unikalus vartotojo ID MBS sistemoje
     *    'account_msisdn' - Apmokestinamo vartotojo telefono numeris (MSISDN), tarptautiniu formatu „370xxxyyyyy“ 
     *    'account_ident' -  WAP vartotojo identifikatorius operatoriaus sistemoje. Unikalus kiekvieno operatoriaus sistemoje. 
     *    'account_type' - Apmokestinamo vartotojo mokėjimo tipas (prepaid, postpaid, unknwon, testing)
     *    'price' - Apmokestinimo suma, nurodoma centais
     *    'tariff' - Apmokestinimo suma operatoriaus sistemoje, nurodoma centais
     *    'currency' - Apmokestinimo valiuta (ISO 4217 standartu, EUR, LTL, LVL ir tt.)
     *    'mt_type' - Vartotojo informavimo tipas (SMS, WapPush)
     *    'mt_text' - Siunčiamo SMS tekstas (jei WapPush, nuorodos antraštė)
     *    'mt_url' - Jei siuntinio tipas WapPush, siunčiamo WapPush nuoroda
     *    'status' - Apmokestinimo transakcijos pagrindinė būsena (status), kuri nusako apmokėjimo sėkmingumą.commit
     *    'state' - Apmokestinimo transakcijos papildoma būsena (state), kuri patikslina nepavykusio apmokėjimo priežastį (galimos reikšmės aprašytos prie metodo getStatusDescription)
     *    'options' - Papildomi apmokestinimui naudoti parametrai, pvz. sesijos ID
     * )
     * </code>
     *
     * @param int $transaction_id Tikrinamos tranzakcijos ID
     * 
     * @return struct|boolean Sėkmės atvėju gražinama tranzakcijos duomenų struktūra (@see self::getTransaction()), nesėkmės false
     * 
     */
    function getTransaction($transaction_id) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);
        
        if (empty($this->transactions[$transaction_id])) {
            $this->initSoap(MBS_DIRECT_SOAP_URL);
            $transaction = $this->_SOAPCall('MBSGetTransaction', array(
                'transaction_id' => (int)$transaction_id,
            ));
	        if ($this->isError()) {
	            return false;
	        }

	        $this->transactions[$transaction_id] = $transaction;
        }
        
        return $this->transactions[$transaction_id];
    }
    
    /**
     * Grąžinama tranzakcija transaction_id jeigu jos būsena yra lygi norimam $status
     *
     * @param int $transaction_id Tikrinamos tranzakcijos ID
     * @param string $status Tranzakcijos būsena Apmokestinimo transakcijos pagrindinė būsena (status), kuri nusako apmokėjimo sėkmingumą 
     * @param string $state (nebūtina) Apmokestinimo transakcijos papildoma būsena (state), kuri patikslina nepavykusio apmokėjimo priežastį
     *
     * @return array|boolean Sėkmės atvėju gražinama tranzakcijos duomenų struktūra (@see @see self::getTransaction()), nesėkmės false
     */
    function getTransactionInStatus($transaction_id, $status, $state=null) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);
        
        $this->initSoap(MBS_DIRECT_SOAP_URL);
        $transaction = $this->_SOAPCall('MBSGetTransactionInState', array(
            'transaction_id'=> (int)$transaction_id,
            'status'        => $status,
            'state'         => $state,
        ));
        if ($this->isError()) {
            return false;
        }
        
        $this->transactions[$transaction_id] = $transaction;
        return $transaction;     
    }
    
    /**
     * Apmokestinimo tvirtinimas.
     *
     * Grąžinama ar pavyko patvirtinti tranzakciją. Skirtas parodyti, kad veiksmai su tranzakcija pilnai baigti. 
     * Sukuriant tranzakciją vartotojo pinigai nuskaičiuojami( rezervuojami).
     * Suteikus vartotojui paslaugą reikia patvirtinti, kad pinigai būtu galutinai nuskaičiuoti.
     *
     * @param int $transaction_id Tvirtinamos tranzakcijos ID
     * 
     * @return boolean Jeigu veiksmas nepavyko rezultatas false jei viskas sėkmingai pavyko grąžinama true.
     */
    function commitTransaction($transaction_id) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);
        
        unset($this->transactions[$transaction_id]);
        
        $this->initSoap(MBS_DIRECT_SOAP_URL);
        return $this->_SOAPCall('MBSCommitTransaction', array(
            'transaction_id' => (int)$transaction_id,
        ));
    }
    
    /**
     * Atšaukiamas vartotojo apmokestinimas, vartotojui pinigai grąžinami.
     *
     * Šį veiksmą reikia atlikti tuo atveju jeigu nepavyko suteikti vartotojui paslaugos pvz.: nepavyko pristatyti turinio vartotojui.
     *
     * <b>!SVARBU! Nevisiems operatoriams galima atlikti šį veiksmą.</b>
     *
     * @param int $transaction_id Atšaukiamos tranzakcijos ID
     * 
     * @return boolean Jeigu veiksmas nepavyko rezultatas false jei viskas sėkmingai pavyko grąžinama true.
     */
    function rollBackTransaction($transaction_id) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);
        
        unset($this->transactions[$transaction_id]);
        
        $this->initSoap(MBS_DIRECT_SOAP_URL);
        return $this->_SOAPCall('MBSRollbackTransaction', array(
            'transaction_id'    => (int)$transaction_id,
        ));
    }
    
    /**
     * Metodas naudojamas apmokestinimo ataskaitos duomenims nuskaityti.
     * Patikrinama ar apmokestinimo ataskaita atsiųsta iš MBS.
     * Nustatomi ir užkraunami apmokestinimo įvykio (tranzakcijos) duomenys.
     * 
     * @return struct|boolean Sėkmės atvėju gražinama tranzakcijos duomenų struktūra (@see self::getTransaction()), nesėkmės false
     */
    function getBillInformation() {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);

        // Check is identify data passed from GET
        if( !empty($_GET['transaction_id']) ) {
            // Check for correct service_id
            if ( empty($_GET['service_id']) || $_GET['service_id']!=$this->service_id ) {
                $this->setError('Direct service not set or wrong service_id');
                return false;
            }
            
            // Checking for correct security signature
            if( MBS_BILL_VERIFY_LEVEL=='NONE' ) {
                $verified = true;
            } else if( MBS_BILL_VERIFY_LEVEL=='LOW' ) {
                $verified = $this->verifyArrayLow($_GET);
            } else {
                $verified = $this->verifyArrayHigh($_GET);
            }
            if( !$verified ) {
                $this->setError('Unauthorise access');
                return false;
            }
            
            // Identified information
            $transaction = array(
                'transaction_id' => $_GET['transaction_id'],
                'service_id' => $_GET['service_id'],
                'ident' => $_GET['ident'],
	            'country' => $_GET['country'],
	            'operator' => $_GET['operator'],
	            'account_id' => $_GET['account_id'],
	            'account_msisdn' => $_GET['account_msisdn'],
	            'account_ident' => $_GET['account_ident'],
	            'account_type' => $_GET['account_type'],
	            'price' => $_GET['price'],
                'tariff' => $_GET['tariff'],
	            'currency' => $_GET['currency'],
                'status' => $_GET['status'],
                'state' => $_GET['state'],
            );
            $this->debug('Transaction data is '.var_export($transaction, true), DEBUG_LEVEL_NORMAL);
            
            return $transaction;
        
        // No billing information in GET parameters
        } else {
            $this->setError('No billing transaction');
        }
    }
    
    /**
     * Patikrinama ar už tranzakciją yra sumokėta.
     *
     * Užmokėta tranzakcija yra jeigu sutampa paslaugos identifikatorius bei sukurto objekto ir tranzakcijos būsena yra "commit".
     *
     * @param int $transaction_id Tikrinamos tranzakcijos ID
     * @return boolean
     *
     * @see getStatusDescription
     */
    function isBilled($transaction_id) {
        $transaction = $this->getTransaction($transaction_id);
        if ($this->isError()) {
            return false;
        }
        
        // Checking is the same billing service for witch made billing.
        if ( $transaction['service_id'] != $this->service_id ) {
            $this->setError("Billing for wrong service {$transaction['service_id']}, expected {$this->service_id}");
            return false;
        }
        
        if ($this->transactions[$transaction_id]['status']=='commit') {
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Tikrinama ar tranzakcija sėkmingai baigta ir ar neįvyko klaidų tikrinant tranzakciją.
     *
     * @param int $transaction_id Tikrinamos tranzakcijos ID
     * @return boolean
     */
    function isCommited($transaction_id) {
        $transaction = $this->getTransaction($transaction_id);
        if ($this->isError()) {
            return false;
        }
                
        if ( in_array($transaction['state'], array('commit', 'auto_commit')) ) {
            return true;
        } else {
            return false;
        }
    }
    
}

?>