<?php
/**
 * Identifikavimo klasės failas
 *
 * @package MBSLib
 * @author Liutauras Rickus <lr@vero.lt>
 * @author Valdas Petrulis <vpe@vero.lt>
 */

require_once('MBSBase.class.php');

/**
 * Identifikavimo klasė skirta WAP vartotojo identifikavimui MBS sistemoje
 *
 * Naudojimo pavyzdys:
 * {@example examples/simple/wap_identify.php}
 * {@example examples/simple/wap_register.php}
 * {@example examples/simple/wap_register_soap.php}
 * {@example examples/simple/wap_unregister_soap.php}
 *
 * @package MBSLib
 *
 */
class Identify extends MBSBase {

    /**
     * Rezervuoti parametrai, kuriuos naudoja MBS ir nebus naudojami kaip paildomi parametrai
     * 
     * @access private
     * @var array
     */
    var $reserved = array (
        'i', 'a', 't', 's', 's1', 's2', 'op', 'oi', 'om', 'sid', 'al', 'ac', 'si', 'sr', 'srs', 'u', 'k'
    );

    /**
     * Identifikavimo paslaugos identifikatorius
     *
     * @access protected
     *
     * @var int
     */
    var $service_id = 0;

    /**
     * Vartotojo informacija
     * 
     * @access protected
     *
     * @var array
     */
    var $user_info = array();

    /**
     * Identifikavimo klases konstruktorius
     *
     * Konstruktoriui paduodami parametrai paslaugos identifikatorius $service_id ir paslaugos
     * slaptas raktas $secret_key. Parametras $use_session skirtas nurodyti ar Identifikuoto
     * vartotojo duomenis saugoti sesijoje.<br/>
     * 
     * Nustatoma ar vartotojo grąžintas iš MBS po identifikavimo. Jei taip, vartotojo duomenys užkraunami.<br/>
     *
     * Jeigu nustatytas savybė use_session po vartotojo nustatymo duomenis patalpinami į sesiją.
     * Vartotojo duomenis atstatomi iš sesijos, jei nėra persiustas iš MBS.<br/>
     *
     * Parametras $back_url nurodo, kur bus nukreiptas vartotojas po identifikavimo.
     * Šį parametrą galima nustatyti konstruktoriuje arba metodo setBackUrl pagalba.<br/>
     *
     * @param int $service_id Identifikavimo paslaugos ID
     * @param string $secret_key Identifikavimo paslaugos slaptas raktas
     * @param boolean[optional] $use_session Ar turi buti naudojama sessija vartotojo duomenims saugoti
     * @param string[optional] $back_url URL į kurį tūrėtų būti gražintas vartotojas
     *
     * @return Identify
     *
     * @see _getUserInformation
     */
    function Identify($service_id, $secret_key, $use_session = true, $back_url = '') {
    	if (!empty($service_id)) {
    		$this->service_id = (int)$service_id;
    	} else {
    		$this->setError("Identify service ID not set");
    	}
    	if (!empty($secret_key)) {
    		$this->setSecretKey($secret_key);
    	} else {
    		$this->setError("Identify secret key (password) not set");
    	}
    	if (!empty($back_url)) {
    		$this->setBackUrl($back_url);
    	}

    	$this->setSessionUse($use_session);
    	if( in_array(MBS_IDENTIFY_TYPE, array('REMOTE', 'BOTH')) ) {
    		$this->initSoap(MBS_IDENTIFY_SOAP_URL);
    	}

    	$this->_getUserInformation();
    }
    
    /**
     * Inicijuoti vartotojo identifikavimą
     *
     * Vykdomas vartotojo identifikavimas. Vartotojas persiunčiamas į MBS identifikavimui.
     * Sėkmingai įvykdžius šią funkciją programos veikimas nutraukiamas.<br/>
     *
     * Parametras $trans privalo būti netuščias. Jo paskirtis susieti vartotoją išėjusį iš partnerių sistemos
     * su vartotoju grįžusiu po identifikavimo. Dažniausia naudojamas laikyti sesijos
     * identifikatorių(tačiau neprivalomai). Šis parametras gražinamas GET[‚t‘] būdu.<br/>
     *
     * Parametras $additional vienmatis masyvas, tai parametrai kurie bus gražinami kartus vartotojo
     * duomenimis po identifikavimo GET‘e.<br/>
     *
     * <b>$addinional parametruose negalima naudoti rezervuotu parametrų 'i', 'a', 't', 's1', 's2', 'op', 'oi', 'sid', 'al', 'ac', 'u', 'k'.</b>
     *
     * Jeigu nustatytas $encode  į TRUE vadinasi nuorodoje visus & keičiame į &amp; tai daroma
     * išvedant nuorodą WML puslapiuose.<br/>
     *
     * @param string $trans Identifikavimo tranzakcija
     * @param array $additional Papildomi parametrai, kurie bus prideti prie identifikavimo URL
     * @param boolean $encode Ar pakeisti simboli '&' į '&amp;' ( be kabučių )
     * 
     * @return boolean
     */
    function MakeIdentify($trans, $additional = array(), $encode = false) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        
        // Identification with remote function needed
        if( in_array(MBS_IDENTIFY_TYPE, array('REMOTE', 'BOTH')) ) {
        	$this->debug('REMOTE identification will be used', DEBUG_LEVEL_NORMAL);
        	
        	$redirect_url = $this->MakeIdentifyRemote($this->service_id, $_SERVER['REMOTE_ADDR'], getallheaders(), $_SERVER['HTTP_COOKIE'], 'http://'.$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'], $_SERVER['HTTP_USER_AGENT']);
        	// Error happened
        	if( $this->isError() ) {
        		return false;
        	// Remote identification asks redirect
        	} else if( !empty($redirect_url) ) {
        		header('Location: '.$redirect_url);
        		exit();
        	// Remote identitication was successful
	    	} else if( $this->getAccount()>0 ) {
	    		// Set that identify process is done
        		$this->user_info['is_identified'] = true;
        		
        		// Set data to session
		        if ( $this->use_session ) {
		            $_SESSION[__CLASS__.$this->suffix] = $this->user_info;
		            $this->debug('New to session', DEBUG_LEVEL_HIGH);
		        }
	    		return true;
	    	}
        }
        
        // Identification with redirect needed
        if( in_array(MBS_IDENTIFY_TYPE, array('REDIRECT', 'BOTH')) ) {
        	$this->debug('REDIRECT identification will be used', DEBUG_LEVEL_NORMAL);
        	
        	$redirect_url = $this->MakeIdentifyUrl($trans, $additional);
        	// Error happened
            if( $this->isError() ) {
                return false;
            // Identification asks redirect
            } else if(!empty($redirect_url)) {
	            header('Location: '.$redirect_url);
	            exit();
	        // No redirect URL given
            } else {
	            $this->setError('Empty redirect URL');
	        }
        }
        
        return false;
    }
    
    /**
     * Performs remote WAP browsing user identification.
     * Current user data are given to MBS remotely, then as much as possible account information is returned (country, language, operator, operator ident, operator MSISDN, etc.)
     * 
     * Grąžinamo masyvo struktūra:
     * <code>
     * array(
     *    'visit' => Identification event structure
     *    'url' => URL to which redirect should be performed
     * )
     * </code>
     * 
     * <b>!IMPORTANT! Remote identification works if operator turned on identification in your server</b>
     * 
     * @param integer $service_id Id of identification service
	 * @param IP $account_ip Visiting account IP address
	 * @param array $account_headers Visiting account headers
	 * @param string $account_cookie (optional) Visiting account cookie string
	 * @param Url $account_url (optional) Visiting account URL
	 * @param string $account_user_agent (optional) Visiting account User Agent
	 * 
     * @return string URL is given if MBS requires redirect for further identification
     */
    function MakeIdentifyRemote($service_id=null, $account_ip, $account_headers, $account_cookie=null, $account_url=null, $account_user_agent=null) {
    	$this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
    	
		$return = $this->_SOAPCall(
			'AccountIdentifyVisitInWAP',
			array(
				'requestor'			=> 'MBSLib',
	    		'service_id'		=> $service_id,
			    'account_ip'		=> $account_ip,
			    'account_headers'	=> $this->_prepareParams($account_headers),
			    'account_cookie'	=> $account_cookie,
			    'account_url'		=> $account_url,
			    'account_user_agent'=> $account_user_agent,
			)
		);
    	if (!$this->isError()) {
    		$this->user_info = array(
				'country'       => $return['visit']['country'],
                'language'      => $return['visit']['language'],
                'operatorName'  => $return['visit']['operator_name'],
                'accountID'		=> $return['visit']['account_id'],
                'operatorID'    => $return['visit']['account_ident'],
                'operatorMSISDN'=> $return['visit']['account_phone'],
                'operatorType'  => $return['visit']['account_type'],
        	);
        	$this->debug('New data is '.var_export($this->user_info, true), DEBUG_LEVEL_NORMAL);
        	
        	return $return['url'];
		}
    }

    /**
     * Veikimas identiškas {@link MakeIdentify} tiktai nenutraukia darbo ir nenukreipia vartotojo į MBS,
     * o grąžina suformuotą nuorodą į MBS.
     *
     * @param string   [optional]   $trans      - transaction id
     * @param array    [optional] 	$additional - additional parameters addet to identify link
     *
     * @return string
     *
     * @see MakeIdentify
     */
    function MakeIdentifyUrl($trans=null,  $additional = array()) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $params = array (
            'i' => $this->service_id,
        );
        if (!empty($trans)) {
        	$params['t'] = $trans;
        }
        if (!empty($this->back_url)) {
            $params['u'] = urlencode($this->back_url);
        }

        foreach ( $additional as $key => $value ) {
            if ( !in_array($key, $this->reserved) ) {
                $params[$key] = $value;
            }
        }
        
        // Pasirašomi duomenys
        $params = $this->signArrayLow($params);
        
        $return = $this->formatUrl(MBS_IDENTIFY_URL, $params);
        $this->debug('Identify url is '.$return, DEBUG_LEVEL_LOW);
        return $return;
    }

    /**
     * Nustatoma ar vartotojo grąžintas iš MBS po identifikavimo jei taip, vartotojo duomenys užkraunami.
     * Šis metodas vykdomas konstruktoriaus metu.
     *
     * @access protected
     */
    function _getUserInformation( ) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);
        
        // Check if identify data passed from SESSION
        if ($this->use_session && !empty($_SESSION[__CLASS__.$this->suffix])) {
            $this->user_info = $_SESSION[__CLASS__.$this->suffix];
        }

        // If new account id passed in GET
        if( isset($_GET['a']) ) {
	        // Checking for correct security signature
	        if( MBS_IDENTIFY_VERIFY_LEVEL=='NONE' ) {
                $verified = true;
	        } else if( MBS_IDENTIFY_VERIFY_LEVEL=='LOW' ) {
	        	$verified = $this->verifyArrayLow($_GET);
	        } else {
	        	$verified = $this->verifyArrayHigh($_GET);
	        }
	        if( !$verified ) {
	            $this->setError('Unauthorise access');
	            return false;
	        }
	        
	        // Checking if it is mobile user
            if( $_GET['a']>0 ) {
                $this->debug('Mobile user detected', DEBUG_LEVEL_NORMAL);
            } else {
            	$this->debug('Not mobile user detected', DEBUG_LEVEL_NORMAL);
            }
	        
	        // Check if new identify data passed from identify service
	        $params_missing = array_diff(array('i', 't', 'ac', 'al', 'a'), array_keys($_GET));
	        if( count($params_missing)<=0 ) {
	            $this->user_info = array(
	                'is_identified' => true,
	                'transaction' => $_GET['t'],
	                'country' => $_GET['ac'],
	                'language' => $_GET['al'],
	                'operatorName' => $_GET['op'],
	                'accountID' => (int)$_GET['a'],
	                'operatorID' => $_GET['oi'],
	                'operatorMSISDN' => isset($_GET['om']) ? $_GET['om'] : null,
	                'operatorType' => isset($_GET['ot']) ? $_GET['ot'] : null,
	                'subscriberIdent' => isset($_GET['si']) ? $_GET['si'] : null,
	                'subscriberStatus' => isset($_GET['sr']) ? $_GET['sr'] : null,
	                'subscriberState' => isset($_GET['srs']) ? $_GET['srs'] : null,
	            );
	            $this->debug('New data is '.var_export($this->user_info, true), DEBUG_LEVEL_NORMAL);
	            
	            // Set data to session
	            if ( $this->use_session ) {
	            	$_SESSION[__CLASS__.$this->suffix] = $this->user_info;
	            	$this->debug('New to session', DEBUG_LEVEL_HIGH);
	            }
            // Some of identify data could be refreshed
	        } else if($this->user_info['accountID']==$_GET['a']) {
	        	$this->debug('Missing identify params: '.var_export($params_missing, true), DEBUG_LEVEL_HIGH);
	        	
	        	if( !empty($_GET['ac']) ) {
	        		$this->user_info['country'] = $_GET['ac'];
	        	}
	        	if( !empty($_GET['al']) ) {
	        		$this->user_info['language'] = $_GET['al'];
	        	}
	        	if( !empty($_GET['op']) ) {
	        		$this->user_info['operatorName'] = $_GET['op'];
	        	}
	        	if( !empty($_GET['oi']) ) {
	        		$this->user_info['operatorID'] = $_GET['oi'];
	        	}
	        	if( !empty($_GET['om']) ) {
	        		$this->user_info['operatorMSISDN'] = $_GET['om'];
	        	}
	            if( !empty($_GET['ot']) ) {
                    $this->user_info['operatorType'] = $_GET['ot'];
                }
	        	if( !empty($_GET['si']) ) {
	        		$this->user_info['subscriberIdent'] = $_GET['si'];
	        	}
	        	if( !empty($_GET['sr']) ) {
	        		$this->user_info['subscriberStatus'] = $_GET['sr'];
	        	}
	        	if( !empty($_GET['srs']) ) {
	        		$this->user_info['subscriberState'] = $_GET['srs'];
	        	}
	        	$this->debug('Refreshed data is '.var_export($this->user_info, true), DEBUG_LEVEL_NORMAL);
	        	
	        	// Set data to session
                if ( $this->use_session ) {
                    $_SESSION[__CLASS__.$this->suffix] = $this->user_info;
                    $this->debug('Partial refresh to session', DEBUG_LEVEL_HIGH);
                }
	        }
        }
    }

    /**
     * Anuliuojami vartotojo duomenys saugomi sesijoje.
     *
     * @deprecated
     *
     */
    function unsetUserData() {
        $this->unsetAccountData();
    }
    
    /**
     * unset user data in session
     *
     */
    function unsetAccountData(){
        unset($_SESSION[__CLASS__.$this->suffix]);
        unset($this->user_info);
    }

    /**
     * Gražinamas masyvas identifikuoto vartotojo duomenų.
     * 
     * Gražinamo masyvo struktūra:
     * <code>
     * array(
     *  'accountID' => <Varotojo identifikatorius MBS sistemoje>
     *  'operatorName' => <Vartotojo operatoriaus pavadinimas>
     *  'operatorID' => <Vartotojo Identifikatorius operatoriaus sistemoje, dažniausiai MSISDN ( telefono numeris )>
     *  'language' => <Vartotojo kalbos kodas ISO 639-1 standartu>
     *  'country' => <Vartotjo šalies kodas ISO 3166-1 alpha-2 standartu>
     *  ...
     *  ...
     * )
     * </code>
     *
     * @deprecated
     *
     * @return array Vartotojo informacija
     */
    function getUserInfo () {
        return $this->getAccountInfo();
    }
    
    /**
     * Gražinamas masyvas identifikuoto vartotojo duomenų.
     * 
     * Gražinamo masyvo struktūra:
     * <code>
     * array(
     *  'accountID' => <Varotojo identifikatorius MBS sistemoje>
     *  'operatorName' => <Vartotojo operatoriaus pavadinimas>
     *  'operatorID' => <Vartotojo Identifikatorius operatoriaus sistemoje, dažniausiai MSISDN ( telefono numeris )>
     *  'language' => <Vartotojo kalbos kodas ISO 639-1 standartu>
     *  'country' => <Vartotjo šalies kodas ISO 3166-1 alpha-2 standartu>
     *  ...
     *  ...
     * )
     * </code>
     *
     * @return array Vartotojo informacija
     */
    function getAccountInfo(){
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info;
    }

    /**
     * Gražinamas vartotojo identifikatorius MBS sistemoje (account_id). Jei gražinama reikšmė 0 vadinasi vartotojo nepavyko identifikuoti.
     *
     * @return int Jei vartotojas identifikuotas gražina > 0, jei neidentifikuotas 0
     */
    function getAccount( ) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return (int)$this->user_info['accountID'];
    }

    /**
     * Gražinamas vartotojo operatoriaus pavadinimas. Pvz:. bite_lt, bite_lv, tele2_ee, omnitel_lt ir t.t.
     *
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     */
    function getOperatorName() {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['operatorName'];
    }

    /**
     * Gražinama vartotojo operatoriaus identifikatorius. Dažniausia tai būna vartotojo telefono numeris.
     *
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     *
     * @deprecated
     */
    function getOperatorId() {
        return $this->getAccountIdent();
    }
    
    /**
     * Gražinama vartotojo operatoriaus identifikatorius. Dažniausia tai būna vartotojo telefono numeris.
     *
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     */
    function getAccountIdent(){
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['operatorID'];
    }

    /**
     * Gražina vartotojo telefono numerį
     * 
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     *
     * @deprecated
     */
    function getOperatorMSISDN() {
        return $this->getAccountMSISDN();
    }

    /**
     * Gražina vartotojo telefono numerį
     * 
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     */
    function getAccountMSISDN(){
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['operatorMSISDN'];
    }
    
    /**
     * Gražina vartotojo mokėjimo tipą (prepaid, postpaid, testing, unknown)
     * 
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     *
     * @deprecated
     */
    function getOperatorType() {
        return $this->getAccountType();
    }

    /**
     * Gražina vartotojo mokėjimo tipą (prepaid, postpaid, testing, unknown)
     * 
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     */
    function getAccountType(){
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['operatorType'];
    }

    /**
     * Gražina vartotoja kalbą ISO 639-1 standartu
     * 
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     *
     * @deprecated
     */
    function getUserLanguage() {
        return $this->getAccountLanguage();
    }


    /**
     * Gražina vartotoja kalbą ISO 639-1 standartu
     * 
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     */
    function getAccountLanguage(){
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['language'];
    }

    /**
     * Gražina vartotoja kalbą ISO 3166-1 alpha-2 standartu
     * 
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     *
     * @deprecated
     */
    function getCountry() {
        return $this->getAccountCountry();
    }


    /**
     * Gražina vartotoja kalbą ISO 3166-1 alpha-2 standartu
     * 
     * <b>Šis parametras gali būti tuščias jeigu paslaugos užsakyme nebuvo nurodyta perduoti.</b>
     *
     * @return string
     */
    function getAccountCountry(){
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['country'];
    }
    
    /**
     * Gražina prenumeratoriaus identifikatorių paslaugoje (jei galima registruotis į vieną paslaugą kelis kartus)
     *
     * @return string
     */
    function getSubscriberIdent(){
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['subscriberIdent'];
    }
    
    /**
     * Gražina prenumeratoriaus statusą paslaugoje
     *
     * @return string
     */
    function getSubscriberStatus(){
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['subscriberStatus'];
    }
    
    /**
     * Gražina prenumeratoriaus tikslesnį statusą paslaugoje
     *
     * @return string
     */
    function getSubscriberState(){
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['subscriberState'];
    }

    /**
     * Gražinamas parametras, kuris buvo nustatytas siunčiant į MBS identifikavimo sistemą.
     *
     * @return string
     */
    function getTransaction() {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
        return $this->user_info['transaction'];
    }

    /**
     * Patikrinama ar vartotoją buvo bandoma nustatyti. Jei buvo bandoma ir nepavyko tai metodas getAccount grąžins 0.
     *
     * @return boolean
     *
     * @see getAccount
     */
    function isIdentified() {
        return ( !empty($this->user_info['is_identified']) ? true : false );
    }
}
?>