<?php
/**
 * Wap apmokėjimų klasės failas
 * 
 * @package MBSLib
 * @author Liutauras Rickus <lr@vero.lt>
 */

require_once('MBSBase.class.php');

/**
 * WAP apmokėjimų klasė
 * 
 * Naudojimo pavyzdžiai:
 * {@example examples/simple/wap_bill.php}
 * {@example examples/simple/wap_bill_soap.php}
 * 
 * @package MBSLib
 * @author Liutauras Rickus <lr@vero.lt>
 * 
 */
class MBS extends MBSBase  {
    
    /**
     * Paslaugos identifikatorius. Apmokestinimo paslaugos, kuri turi konkrečias kainas
     * 
     * @access protected
     *
     * @var int
     */
    var $service_id = 0;
    
    /**
     * Paslaugos kanalo identifikatorius.
     * 
     * @access protected
     * 
     * @var int
     */
    var $channel_id = 0;
    
    /**
     * Vidiniai transakcijų duomenys
     *
     * @access protected
     *
     * @var array
     */
    var $transaction_data = array();
    
    /**
     * Wap paslaugų apmokėjimų klasės konstruktorius
     *
     * Konstruktoriu nustatome paslaugos identifikatorių $service_id ir kanalo identifikatorių $channle_id.<br/>
     *
     * Jeigu $back_url nenurodyta gražinimui bus naudojami URL nurodyti paslaugos aprašymuose.
     * Papildomai galima nurodyti $back_url, tokiu atvėju vartotojas iš Vero MBS sistemos ( po apmokęstinimo )
     * bus gražinamas nurodytą nuorodą.<br/>
     *
     *  Taip pat nuorodoje galima naudoti žymenis:
     *    # %t arba %{transaction_id}% - pakeistas į naudojamos transakcijos ID
     *    # %s or %{service_id}% - pakeistas į naudojamos paslaugos(service) ID
     *    # %c or %{channel_id}% - pakeistas į naudojamo kanalo (channel) ID
     *    # %p or %{transaction_price} - pakeistas į paslaugos kainą
     *    # %cur or %{transaction_currency} - pakeistas į paslaugos kainos valiutą
     *    # %o or %{operator_id}%- pakeistas į vartotojo operatoriaus (operator) ID
     *    # %a or %{account_id}%- pakeistas į vartotojo identifikatorių (account) ID
     *    # %n or %{operator_name}%	- pakeistas į vartotojo operatoriaus pavadinimą (pvz.: bite_lt, tele2_lv, ect.)
     *
     * Šie žymenis bus pakeisti atitinkamomis reikšmėmis. Pvz.: $back_url nustatome į
     * <b>http://example.lt?trans=%t&s=%s&service=%{service_id}%&a=te</b>
     *
     * Įvykdytos transakcijos id buvo <b>2456</b> naudotos paslaugos (service_id) <b>1</b> tokiu atveju vartotojas bus gražintas nuoroda:
     *
     * <b>http://example.lt?trans=2456&s=1&service=1&a=te</b>
     *
     * @param int $service_id Wap paslaugos identifikatorius
     * @param int $channel_id (nebūtina) Kanalo identifikatorius
     * @param string $back_url (nebūtina) URL kuriuo gražinti vartotoja po registracijos
     * @param string $secret_key (nebūtina) Slaptas žodis, kuris naudojamas saugumo parašui
     * 
     * @return MBS
     */
    function MBS($service_id, $channel_id=null, $back_url=null, $secret_key=null) {
    	if (!empty($service_id)) { 
    		$this->service_id = (int)$service_id; 
    	} else { 
    		$this->setError('Service ID is not set');
    	}
    	if (!empty($channel_id)) {
    		$this->channel_id = $channel_id;
    	}
    	if (!empty($back_url)) {
    		$this->setBackUrl($back_url);
    	}
    	if (!empty($secret_key)) {
    		$this->setSecretKey($secret_key);
    	}
    	
    	$this->debug('MBS class created. Params service_id: ' . $this->service_id . ', channel_id: ' . $this->channel_id, DEBUG_LEVEL_HIGH);
    }
    
    /**
     * Vykdomas vartotojo apmokestinimas.
     *
     * Atlikus šį metodą vartotojas bus nukreiptas į Vero sistemą ir programos darbas nutrauktas.<br/>
     *
     * Parametras $params nusako kokie parametrai bus gražinami atgal
     * (žiūrėti į {@link MBS konstruktorių} apie $back_url.)<br/>
     *
     * Parametras $lang nurodo kokia kalba vartotojas matys pranešimus Vero sistemoje.
     * Jei paslaugos parametras rodyti klausymą nurodytas. Tai šis parametras nurodo kokia kalba
     * rodyti klausymą.<br/>
     *
     * Parametras $ident nenaudojamas, bet yra numatytas sistemoje. Ateityje gali būti
     * tikrinama ar tas pats vartotojas už tokį ident jau mokėjo valandos bėgyje.
     * Kad nebūtu daug kartiniai  vartotojo apmokestinimai.
     *
     * @param array $params Parametrai kuriuos reikia pridėti prie atgalinio URL
     * @param string $lang (nebūtina) Vartotojo kalbos kodas ISO 639-1 standartu ( šia kalba bus rodomas klauimas )
     * @param string $ident (nebūtina) Unikalus paslaugos identifikatorius ( tarkim pardavinejat melodijas, tai gali būti melodijos ID ar pavadinimas )
     * @param integer $price (nebūtina) Apmokestinimo kaina, reikia nurodyti, kai paslauga turi kelias kainas (valiutos 1/100 dalimis - centais)
     * @param string $currency (nebūtina) Kainos valiuta
     */
    function bill($params, $lang=null, $ident=null, $price=null, $currency=null) {
    	$this->debug(__FUNCTION__.' was started', DEBUG_LEVEL_HIGH );
    	
    	$params = $this->prepareParams($params);
        $transaction_id = $this->createTransaction($this->service_id, $this->channel_id, $lang, $params, $ident, $price, $currency);
        if (!$this->isError()) {
            header('Location: '.MBS_BILL_USER_REDIRECT_URL.'?transaction='.$transaction_id);
            exit();
        } else {
            return false;
        }
    }
    
    /**
     * Vartotojo WAP apmokestinims redirect būdu. Žiūrėkite {@link Bill} platesniam aprašymui. 
     * 
     * @param array $params (nebūtina) Papildomi parametrai, kurie bus gražinami atgal
     * @param string $ident (nebūtina) Unikalus pirkimo identifikatorius partnerio sistemoje
     * @param integer $price (nebūtina) Apmokestinimo kaina, reikia nurodyti, kai paslauga turi kelias kainas (valiutos 1/100 dalimis - centais)
     * @param string $currency (nebūtina) Kainos valiuta
     * @param string $url_false (nebūtina) URL, į kurį nukreipiama po nesėkmingo apmokestinimo
     * 
     * @see Bill
     */
    function billRedirect($params=array(), $ident=null, $price=null, $currency=null, $url_false=null) {
        $this->debug(__FUNCTION__.' was started', DEBUG_LEVEL_HIGH );

        // Formuojami apmokestinimo parametrai
        $params['s']  = $this->service_id;
        $params['i']  = $ident;
        if( is_numeric($price) ) {
            $params['p'] = $price;
            $params['pc'] = $currency;
        }
        $params['u'] = $this->back_url;
        if( !empty($url_false) && $url_false!=$this->back_url ) {
            $params['un'] = $url_false;
        }
        
        // Pasirašomi duomenys
        $params = $this->signArrayLow($params);
        
        // Vartotojas siunčiamas į suformuotą URL adresą
        header('Location: '.$this->formatUrl(MBS_BILL_REDIRECT_URL, $params));
        exit();
    }
    
    /**
     * Nustatoma ar vartotojo grąžintas iš MBS po apmokestinimo jei taip, apmokestinimo duomenys užkraunami.
     * 
     * @return array Billing transaction data
     */
    function getBillInformation() {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);

        // Check is identify data passed from GET
        if( !empty($_GET['t']) ) {
            // Check for correct service_id
            if ( empty($_GET['s']) || $_GET['s']!=$this->service_id ) {
                $this->setError('WAP service not set or wrong service_id');
                return false;
            }
        	
            // Checking for correct security signature
            if( MBS_BILL_VERIFY_LEVEL=='NONE' ) {
                $verified = true;
            } else if( MBS_BILL_VERIFY_LEVEL=='LOW' ) {
                $verified = $this->verifyArrayLow($_GET);
            } else {
                $verified = $this->verifyArrayHigh($_GET);
            }
            if( !$verified ) {
                $this->setError('Unauthorise access');
                return false;
            }
            
            // Identified information
            $transaction = array(
                'transaction_id' => $_GET['t'],
                'service_id' => $_GET['s'],
                'account_id' => $_GET['a'],
                'ident' => $_GET['i'],
                'status' => $_GET['r'],
                'state' => $_GET['rs'],
            );
            $this->debug('Transaction data is '.var_export($transaction, true), DEBUG_LEVEL_NORMAL);
            
            return $transaction;
        
        // No billing information in GET parameters
        } else {
            $this->setError('No billing transaction');
        }
    }
    
    /**
     * Pagal $transaction_id gražinama tranzakcijos informacija.
     *
     * Jeigu atitinkamos tranzakcijos nėra arba tranzakcijos parametrai - paslaugos ir kanalo arba paslaugos, kainos ir valiutos
     * identifikatoriai nesutampa su reikalaujamos tranzakcijos, gražinama false reikšmė.<br/>
     *
     * Grąžinamo masyvo struktūra 1.0 versijai:
     * <code>
     * array(
     *    'transaction_id' => <tranzakcijos ID>
     *    'service_id' => <paslaugos ID>
     *    'channel_id' => <kanalo ID>
     *    'account_id' => <vartotojo identifikatorius (account_id Vero MBS sistemoje)>
     *    'state' => <tranzakcijos {@link getStatusDescription būsena} (galimos reikšmės aprašytos prie metodo getStatusDescription)>
     * )
     * </code>
     *
     * Grąžinamo masyvo struktūra 1.1 versijai:
     * <code>
     * array(
     *    'transaction_id' => <tranzakcijos ID>
     *    'service_id' => <paslaugos ID>
     *    'account_id' => <vartotojo identifikatorius (account_id Vero MBS sistemoje)>
     *    'price' => <paslaugos kaina centais>
     *    'currency' => <paslaugos kainos valiutos kodas>
     *    'state' => <tranzakcijos {@link getStatusDescription būsena} (galimos reikšmės aprašytos prie metodo getStatusDescription)>
     * )
     * </code>
     *
     * <b>!SVARBU!Norint įsitikinti, kad vartotojas neapgaudinėja reiktu tikrinti ar vartotojo account_id tokia kaip tranzakcijos.</b>
     *
     * @param int $transaction_id
     * @return array|boolean Sėkmės atvėju gražinamas array, nesėkmės false
     * 
     */
    function getTransaction( $transaction_id ) {
    	$this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
    	
    	if (empty($this->transaction_data[$transaction_id])) {
			$info = array(
				'transaction_id' => (int)$transaction_id,	// transction_id
			);
			$this->initSoap(MBS_BILL_SOAP_URL);
			$return = $this->_SOAPCall('MBSGetTransaction', $info);
			
			if (!$this->isError()) {
				// Checking is the same billing service and channel for witch made billing.
                if ( $return['service_id'] != $this->service_id ) {
                    $this->setError("Billing for wrong service {$return['service_id']}, expected {$this->service_id}");
                }
                if( MBS_BILL_SOAP_VERSION=='1.0' && $return['channel_id'] != $this->channel_id ) {
                	$this->setError("Billing for wrong channel {$return['channel_id']}, expected {$this->channel_id}");
                }
				
				$this->transaction_data[$transaction_id] = $return;
			}
    	}
		
		return $this->transaction_data[$transaction_id];
    }
    
    /**
     * Patikrinama ar už tranzakciją yra sumokėta.
     *
     * Užmokėta tranzakcija yra jeigu sutampa paslaugos identifikatorius ir kanalas su
     * sukurto objekto ir tranzakcijos būsena yra "commit".
     *
     * @param int $transaction_id Tranzakcijos identifikatorius
     * @return boolean
     *
     * @see getStatusDescription
     */
    function isBilled($transaction_id) {
    	if (empty($this->transaction_data[$transaction_id])) {
    		$this->getTransaction($transaction_id);
    	}
    	
    	if ($this->isError()) {
    		return false;
    	}
    	
    	if ($this->transaction_data[$transaction_id]['status'] == 'commit') {
    		return true;
    	}
    	return false;
    }
    
    /**
     * Tikrinama ar tranzakcija sėkmingai baigta ir ar neįvyko klaidų tikrinant tranzakciją.
     *
     * @param int $transaction_id Tranzakcijos identifikatorius
     * @return boolean
     */
    function isCommited($transaction_id) {
    	if (empty($this->transaction_data[$transaction_id])) {
    		$this->getTransaction($transaction_id);
    	}
    	
    	if ($this->isError()) {
    		return false;
    	}
    	    	
    	if (in_array( $this->transaction_data[$transaction_id]['state'], array('commit', 'auto_commit') ) ) {
    		return true;
    	}
    	return false;
    }
    
    /**
     * Apmokestinimo tvirtinimas tranzakcijos pabaigimas.
     *
     * Grąžinama ar pavyko patvirtinti tranzakciją. Skirtas parodyti, kad veiksmai su tranzakcija pilnai baigti. 
     * Sukuriant tranzakciją vartotojo pinigai nuskaičiuojami(rezervuojami). Suteikus vartotojui
     * paslaugą reikia patvirtinti, kad pinigai būtu galutinai nuskaičiuoti.<br/>
     *
     * Ateityje ruošiamas padaryti, kad šio veiksmo daryti nereiks ir jeigu tranzakcija nėra baigta
     * "commit", tai po tam tikro laiko automatiškai pritaikomas šis veiksmas ( jeigu tranzakcija ne atšaukta ).
     *
     * @param int $transaction_id Tranzakcijos identifikatorius
     * @return boolean
     */
    function commitTransaction($transaction_id) {
    	$this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);
    	unset($this->transaction_data[$transaction_id]);
    	
		$info = array(
			'transaction_id' => (int)$transaction_id, // transction_id
		);
		$this->initSoap(MBS_BILL_SOAP_URL);
		$return = $this->_SOAPCall('MBSCommitTransaction', $info);
		
		return $return;       	
    }
    
    /**
     * Atšaukiamas vartotojo apmokestinimas vartotojui pinigai gražinami.
     *
     * Jeigu veiksmas nepavyko grąžinamas rezultatas false jei viskas sėkmingai pavyko grąžinama true.
     * Šį veiksmą reikia atlikti tuo atveju jeigu nepavyko suteikti vartotojui paslaugos pvz.:
     * nepavyko pristatyti turinio vartotojui.<br/>
     *
     * <b>!SVARBU! Nevisiems operatoriams galima atlikti šį veiksmą. </b>
     *
     * @param int $transaction_id Tranzakcijos identifikatorius
     * @return boolean
     */
    function rollBackTransaction($transaction_id) {
    	$this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
    	
    	unset($this->transaction_data[$transaction_id]);
		$info = array(
			'transaction_id' 	=> (int)$transaction_id,	// transction_id
		);
		$this->initSoap(MBS_BILL_SOAP_URL);
		$return = $this->_SOAPCall('MBSRollbackTransaction', $info);
		if ($this->isError()) {
			return false;
		}
		return $return;       	
    }
    
    /**
     * Kuriama tranzakcija
     *
     * Veiksmas atliekamas prieš apmokestinimą. Naudojama metode {@link Bill}
     * Vykdoma tiesioginė SOAP užklausa.
     *
     * $options – parametrai kurie paduodami į tranzakciją ir vėliau galim juos pasiimti jeigu
     * pasulaugos kūrimo metu į nuorodas yra įrašyti forma %{my_<parametro vardas>}%. Galima
     * naudoti parametrą url kuris bus nurodo į kokią nuorodą grąžinti po apmokestinimo detaliau
     * aprašyta prie MBS konstruktoriaus.<br/>
     *
     * Norint paduoti toki masyva:
     * <code>
     * array(
     *  'key1'=>'value1',
     *  'key2'=>'value2',
     * )
     * </code>
     * 
     * Reikia jį transformuoti į:
     * <code>
     * array(
     *    array('name'=>'key1', 'value'=>'value1'),
     *    array('name'=>'key2', 'value'=>'value2'),
     * )
     * </code> 
     *
     * @param int $service_id Wap paslaugos identifikatorius
     * @param int $channel_id (nebūtina) Wap paslaugos kanalo identifikatorius
     * @param string $language_ident (nebūtina) Kalbos kodas ISO 639-1 standartu ( šia kalba bus rodomas klausimas )
     * @param array $options (nebūtina) Papildomi parametrai
     * @param string $ident (nebūtina) Unikalus paslaugos identifikatorius ( tarkim pardavinejat melodijas, tai gali būti melodijos ID ar pavadinimas )
     * @param integer $price (nebūtina) Apmokestinimo kaina, reikia nurodyti, kai paslauga turi kelias kainas (valiutos 1/100 dalimis - centais)
     * @param string $currency (nebūtina) Kainos valiuta
     * 
     * @return int|boolean Sėkmės atvėju gražinamas tranzakcijos ID, nesėkmės false
     * 
     */
    function createTransaction($service_id, $channel_id=null, $language_ident=null, $options=array(), $ident=null, $price=null, $currency=null) {
    	$this->debug(__FUNCTION__.' was started', DEBUG_LEVEL_HIGH );

        if (MBS_BILL_SOAP_VERSION == '1.0') {
        	$info = array(
                'service_id'     => $service_id, // id of MBS service which is dedicated for your ring tones
                'channel_id'     => $channel_id, // id of MBS service channel
                'language_ident' => $language_ident, // ISO 639-1 code of language in which agreement to pay question will be shown 
                'options'        => $options, // Extra options that will be returned to you after billing
                'ident'          => $ident, // Unique identifier of your service ( example: ring tone's id)
            );
        } else {
			$info = array(
				'service_id'     => $service_id, // id of MBS service which is dedicated for your ring tones
			    'price'          => $price, // price of MBS service
			    'currency'       => $currency, // currency of MBS service
				'language_ident' => $language_ident, // ISO 639-1 code of language in which agreement to pay question will be shown 
				'options' 		 => $options, // Extra options that will be returned to you after billing
				'ident'          => $ident, // Unique identifier of your service ( example: ring tone's id)
			);
    	}
    	$this->initSoap(MBS_BILL_SOAP_URL);
		$transaction_id = (int)$this->_SOAPCall('MBSCreateTransaction', $info);
		
		return $transaction_id;
    }
    
    /**
     * Grąžinama tranzakcija transaction_id jeigu jos būsena reikšmė yra $state. 
     * 
     * Grąžinamo masyvo struktūra 1.0 versijai:
     * <code>
     * array(
     *    'transaction_id' => <tranzakcijos ID>
     *    'service_id' => <paslaugos ID>
     *    'channel_id' => <kanalo ID>
     *    'account_id' => <vartotojo identifikatorius (account_id Vero MBS sistemoje)>
     *    'state' => <tranzakcijos {@link getStatusDescription būsena} (galimos reikšmės aprašytos prie metodo getStatusDescription)>
     * </code>
     * 
     * Grąžinamo masyvo struktūra 1.1 versijai:
     * <code>
     * array(
     *    'transaction_id' => <tranzakcijos ID>
     *    'service_id' => <paslaugos ID>
     *    'account_id' => <vartotojo identifikatorius (account_id Vero MBS sistemoje)>
     *    'price' => <paslaugos kaina centais>
     *    'currency' => <paslaugos kainos valiutos kodas>
     *    'state' => <tranzakcijos {@link getStatusDescription būsena} (galimos reikšmės aprašytos prie metodo getStatusDescription)>
     * </code>
     *
     * @param int $transaction_id Tranzakcijos identifikatorius
     * @param string state Tranzakcijos būsena
     *
     * @return array|boolean Sėkmės atvėju gražinamas array, nesėkmės false
     */
    function getTransactionInState( $transaction_id, $state ) {
    	$this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );
    	
		$info = array(
			'transaction_id' 	=> (int)$transaction_id,	// transction_id
			'state'				=> $state,					// expectit state
		);
		$this->initSoap(MBS_BILL_SOAP_URL);
		$return = $this->_SOAPCall('MBSGetTransactionInState', $info);
		
		return $return;    	
    }
}

?>