<?php
/**
 * SBBS apmokėjimų klasės failas
 *
 * @package MBSLib
 * @author Liutauras Rickus <lr@vero.lt>
 * @author Valdas Petrulis <vpe@vero.lt>
 * @author Karolis Tamutis <kt@vero.lt>
 */
 
require_once('MBSBase.class.php');

/**
 * Vartotojų valdymo Vero SBBS(MBS) prenumeratų sistemoje
 *
 * Kuriant objektą reikia naudoti apmokestinimo wap paslaugos identifikatorių ir kanalą, nes registruojant
 * į paslaugą naudojamas wap apmokestinimo parametrai. Tačiau atlikus reigistraciją vartotojas yra
 * registruojamas į prenumeratinę paslaugą, kurios identifikatorius kitas<br/>
 *
 * Registravimo į prenumeratą pavyzdžiai:
 * {@example examples/simple/wap_register.php}
 * {@example examples/simple/wap_register_soap.php}
 * 
 * Išregistravimo iš prenumeratos pavyzdžiai:
 * {@example examples/simple/wap_unregister.php}
 * {@example examples/simple/wap_unregister_soap.php}
 * 
 * Ataskaitų išorinei sistemai pavyzdžiai:
 * {@example examples/simple/notify_subscription.php}
 * {@example examples/simple/notify_sbbs.php}
 *
 * @package MBSLib
 * @author Liutauras Rickus <lr@vero.lt>
 *
 */
class SBBS extends MBSBase  {

    /**
     * Paslaugos identifikatorius
     *
     * @access protected
     *
     * @var int
     */
    var $service_id = 0;

    /**
     * Kanalo identifikatorius
     * @access protected
     *
     * @var int
     */
    var $channel_id = 0;

    /**
     * Vartotojo informacija
     * 
     * @access protected
     *
     * @var array
     */
    var $user_info = array();

    /**
     * SBBS klasės konstruktorius
     *
     * Konstruktoriui nustatome paslaugos identifikatorių $service_id ir kanalo identifikatorių $channle_id.<br/>
     * Jeigu $back_url nenurodyta gražinimui bus naudojami URL nurodyti paslaugos aprašymuose.
     * Papildomai galima nurodyti $back_url. Jeigu jis nurodytas nusiustas vartotojas į Vero sistemą bus
     * gražinamas šia nuorodą iš registracijos. Taip pat nuorodoje galima naudoti žymenis:
     *    # %t arba %{transaction_id}% - pakeistas į naudojamos transakcijos ID
     *    # %s or %{service_id}% - pakeistas į naudojamos paslaugos(service) ID
     *    # %c or %{channel_id}%- pakeistas į naudojamo kanalo (channel) ID
     *    # %o or %{operator_id}%- pakeistas į vartotojo operatoriaus (operator) ID
     *    # %a or %{account_id}%- pakeistas į vartotojo identifikatorių (account) ID
     *    # %n or %{operator_name}%	- pakeistas į vartotojo operatoriaus pavadinimą (pvz.: bite_lt, tele2_lv, ect.)
     *
     * Šie žymenis bus pakeisti atitinkamomis reikšmėmis. Pvz.: $back_url nustatome į
     * <b>http://example.lt?trans=%t&s=%s&service=%{service_id}%&a=te</b><br/>
     *
     * Įvykdytos transakcijos id buvo <b>2456</b> naudotos paslaugos (service_id) <b>1</b> tokiu atveju vartotojas bus gražintas nuoroda:
     * <b>http://example.lt?trans=2456&s=1&service=1&a=te</b><br/>
     *
     * Vartotojo išregistravimas neturi galimybės vartotoją gražinti į $bill_url.
     * Ateityje planuojama tai pataisyti.
     *
     * @param int $service_id <b>WAP registracijos paslaugos</b> identifikatorius
     * @param int $channel_id (nebūtina) Kanalo identifikatorius
     * @param string $back_url (nebūtina) URL kuriuo gražinti vartotoja po registracijos
     * @param string $secret_key (nebūtina) Slaptas žodis, kuris naudojamas saugumo parašui
     *
     * @return SBBS
     */
    function SBBS($service_id, $channel_id=null, $back_url=null, $secret_key=null) {
    	if( !empty($service_id) ) {
            $this->service_id = (int)$service_id;
        } else {
            $this->setError('Service ID not set');
        }
        if( !empty($channel_id) ) {
            $this->channel_id = (int)$channel_id;
        }
        if( !empty($back_url) ) {
            $this->setBackUrl($back_url);
        }
        if( !empty($secret_key) ) {
            $this->setSecretKey($secret_key);
        }
        
        $this->debug('SBBS class created. Params service_id: ' . $this->service_id . ', channel_id: ' . $this->channel_id, DEBUG_LEVEL_HIGH);
    }
	
    /**
     * Vartotojo registravimas į prenumeratą redirect būdu.
     * 
     * @param array $params (nebūtina) Papildomi parametrai, kurie bus gražinami atgal
     * @param string $ident (nebūtina) Unikalus pirkimo identifikatorius partnerio sistemoje
     * @param string $subscriber_ident (nebūtina) Prenumeratoriaus identifikatorius paslaugoje (jei galima registruotis į vieną paslaugą kelis kartus)
     * @param integer $price (nebūtina) Registracijos kaina, reikia nurodyti, kai paslauga turi kelias kainas (valiutos 1/100 dalimis - centais)
     * @param string $currency (nebūtina) Kainos valiuta
     * @param string $url_false (nebūtina) URL, į kurį nukreipiama po nesėkmingos registracijos
     */
    function registerRedirect($params=array(), $ident=null, $subscriber_ident=null, $price=null, $currency=null, $url_false=null) {
        $this->debug(__FUNCTION__.' was started', DEBUG_LEVEL_HIGH);
        
        // Formuojami registravimo parametrai
        $params['s'] = $this->service_id;
        $params['si'] = $subscriber_ident;
        $params['i'] = $ident;
        if( is_numeric($price) ) {
        	$params['p'] = $price;
        	$params['pc'] = $currency;
        }
        $params['u'] = $this->back_url;
        if( !empty($url_false) && $url_false!=$this->back_url ) {
            $params['un'] = $url_false;
        }
        
	    // Pasirašomi duomenys
		$params = $this->signArrayLow($params);
		
		// Vartotojas siunčiamas į suformuotą URL adresą
		header('Location: '.$this->formatUrl(MBS_SUBSCRIPTION_REGISTER_URL, $params));
        exit();
    }
    
    /**
     * Nustatoma ar vartotojo grąžintas iš MBS po registracijos jei taip, apmokestinimo duomenys užkraunami.
     * 
     * @return array Billing transaction data
     */
    function getRegisterInformation() {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);

        // Check is identify data passed from GET
        if( !empty($_GET['t']) ) {
            // Check for correct service_id
            if ( empty($_GET['s']) || $_GET['s']!=$this->service_id ) {
                $this->setError('WAP service not set or wrong service_id');
                return false;
            }
            
            // Checking for correct security signature
            if( MBS_SUBSCRIPTION_VERIFY_LEVEL=='NONE' ) {
                $verified = true;
            } else if( MBS_SUBSCRIPTION_VERIFY_LEVEL=='LOW' ) {
                $verified = $this->verifyArrayLow($_GET);
            } else {
                $verified = $this->verifyArrayHigh($_GET);
            }
            if( !$verified ) {
                $this->setError('Unauthorise access');
                return false;
            }
            
            // Identified information
            $transaction = array(
                'transaction_id' => $_GET['t'],
                'service_id' => $_GET['s'],
                'account_id' => $_GET['a'],
                'ident' => $_GET['i'],
                'subscriber_ident' => $_GET['si'],
                'subscriber_status' => $_GET['sr'],
                'subscriber_state' => $_GET['srs'],
            );
            $this->debug('Transaction data is '.var_export($transaction, true), DEBUG_LEVEL_NORMAL);
            
            return $transaction;
        
        // No billing information in GET parameters
        } else {
            $this->setError('No register transaction found');
        }
    }
    
    /**
     * Vartotojo išregistravimas iš prenumeratos redirect būdu.
     * 
     * @param array $params (nebūtina) Papildomi parametrai, kurie bus grazinami atgal
     * @param string $ident (nebūtina) Unikalus pirkimo identifikatorius partnerio sistemoje
     * @param string $subscriber_ident (nebūtina) Prenumeratoriaus identifikatorius paslaugoje (jei galima registruotis į vieną paslaugą kelis kartus)
     * @param string $url_false (nebūtina) URL, į kurį nukreipiama po nesėkmingos registracijos
     */
    function unregisterRedirect($params=array(), $ident=null, $subscriber_ident=null, $url_false=false) {
        $this->debug(__FUNCTION__.' was started', DEBUG_LEVEL_HIGH);
        
        // Formuojami isregistravimo parametrai
        $params['s'] = $this->service_id;
        $params['si'] = $subscriber_ident;
        $params['i'] = $ident;
        $params['u'] = $this->back_url;
        if( !empty($url_false) && $url_false!=$this->back_url ) {
        	$params['un'] = $url_false;
        }
        
        // Pasirašomi duomenys
        $params = $this->signArrayLow($params);
        
        // Vartotojas siunčiamas į suformuotą URL adresą
        header('Location: '.$this->formatUrl(MBS_SUBSCRIPTION_UNREGISTER_URL, $params));
        exit();
    }
    
    /**
     * Nustatoma ar vartotojo grąžintas iš MBS po išregistravimo jei taip, ivykio duomenys užkraunami.
     * 
     * @return array Unregistration transaction data
     */
    function getUnregisterInformation() {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH);

        // Check is identify data passed from GET
        if( !empty($_GET['t']) ) {
            // Check for correct service_id
            if ( empty($_GET['s']) || $_GET['s']!=$this->service_id ) {
                $this->setError('WAP service not set or wrong service_id');
                return false;
            }
            
            // Checking for correct security signature
            if( MBS_SUBSCRIPTION_VERIFY_LEVEL=='NONE' ) {
                $verified = true;
            } else if( MBS_SUBSCRIPTION_VERIFY_LEVEL=='LOW' ) {
                $verified = $this->verifyArrayLow($_GET);
            } else {
                $verified = $this->verifyArrayHigh($_GET);
            }
            if( !$verified ) {
                $this->setError('Unauthorise access');
                return false;
            }
            
            // Identified information
            $transaction = array(
                'transaction_id' => $_GET['t'],
                'service_id' => $_GET['s'],
                'account_id' => $_GET['a'],
                'ident' => $_GET['i'],
                'subscriber_ident' => $_GET['si'],
                'subscriber_status' => $_GET['sr'],
                'subscriber_state' => $_GET['srs'],
            );
            $this->debug('Transaction data is '.var_export($transaction, true), DEBUG_LEVEL_NORMAL);
            
            return $transaction;
        
        // No billing information in GET parameters
        } else {
            $this->setError('No unregister transaction found');
        }
    }
    
    /**
     * Vykdomas registravimas į prenumeratą.
     *
     * Atlikus šį metodą vartotojas bus nukreiptas į Vero sistemą ir programos darbas nutrauktas.<br/>
     *
     * Parametras $options nusako kokie parametrai bus gražinami atgal žiūrėti į konstruktorių apie
     * $back_url.<br/>
     *
     * Parametras $lang nurodo kokia kalba vartotojas matys pranešimus Vero sistemoje. Jei paslaugos
     * parametras rodyti klausymą nurodytas. Tai šis parametras taip pat nurodo kokia kalba rodyti klausimą. 
     * Kalbos kodas ISO 639-1 standartu.<br/>
     *
     * Parametras $ident Skirtas vienam vartotojui būti registruotam vienoje prenumeratoje kelis kartus naudojant skirtingą šį parametrą.<br/>
     *
     * Parametras $valid_till Skirtas nurodyti kiek laiko po registracijos vartotojui galios prenumerata.<br/>
     *
     * @param array $options Papildomi parametrai, kuriuos MBS pridės prie atgalinio URL po registracijos
     * @param string $lang_id Kalbos kodas ISO 639-1 standartu ( naudojamas rodant klausimą )
     * @param string $ident Papildomas registruoto vartotojo identifikatorius ( kai norima viena vartotoja registruoti į prenumeratą daugiau negu vieną karta ) !Dar neveikia!
     * @param string $valid_till Data, iki kurios galios prenumerata
     *
     */
    function register($options, $lang_id=null, $ident=null, $valid_till=null) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $url = $this->registerUrl($options, $lang_id, $ident, $valid_till);
        if (!empty($url)) {
            header('Location: '.$url);
            exit();
        }

        return false;
    }

    /**
     * Gražinama nuoroda, kurią vartotojas turi būti nukreipiamas, kad vartotojas būtu registruotas į prenumeratą.
     *
     * Parametrai identiški {@link register} metodui.
     *
     * @param array $options Papildomi parametrai, kuriuos MBS pridės prie atgalinio URL po registracijos
     * @param string $lang_id Kalbos kodas ISO 639-1 standartu ( naudojamas rodant klausimą )
     * @param string $ident Papildomas registruoto vartotojo identifikatorius ( kai norima viena vartotoja registruoti į prenumeratą daugiau negu vieną karta ) !Dar neveikia!
     * @param string $valid_till Data, iki kurios galios prenumerata
     * 
     * @return string
     *
     * @see register
     */
    function registerUrl($options, $lang_id=null, $ident=null, $valid_till=null ) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $options = $this->prepareParams($options);
        $return = $this->SubscriptionWapRegister($this->service_id, $this->channel_id, $lang_id, $options, $ident, $valid_till);
        $this->debug( 'Returned: '.var_export($return, true), DEBUG_LEVEL_NORMAL);

        if (!$this->isError() && !empty($return['url'])) {
            return $return['url'];
        }

        return '';
    }

    /**
     * Išregistruojamas vartotojas iš prenumeratos.
     *
     * Išregistravimas vyksta Vero sistemoje, todėl vartotojas siunčiamas į Vero sistema iš registravimui
     * įvykdžius šį metodą programos darbas nutraukiamas.<br/>
     *
     * Vartotojas po išregistravimo nukreipiamas adresu nurodytu prie prenumeratos paslaugos užsakymo.<br/>
     *
     * <b>!SVARBU! Vartotojas nėra grąžinamas į $back_url. </b>
     *
     * @param array $options Papildomi parametrai, kuriuos MBS pridės prie atgalinio URL po registracijos
     * @param string $lang_id Kalbos kodas ISO 639-1 standartu ( naudojamas rodant klausimą )
     * @param string $ident Papildomas registruoto vartotojo identifikatorius ( kai norima viena vartotoja registruoti į prenumeratą daugiau negu vieną karta ) !Dar neveikia!
     * @param string $unregister_reason
     */
    function unregister($options, $lang_id=null, $ident=null, $unregister_reason='') {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $url = $this->unregisterUrl($options, $lang_id, $ident, $unregister_reason);
        if (!empty($url)) {
            header('Location: '.$url);
            exit();
        }
    }

    /**
     * Gražinama nuoroda, kurią vartotojas turi būti nukreipiamas, kad vartotojas būtu išregistruotas iš prenumeratos.
     *
     * Parametrai identiški {@link unregister} metodui.
     * Veikimas identiškas tik gražinamas URL ir darbas nenutraukiamas.
     *
     * @param array $options Papildomi parametrai, kuriuos MBS pridės prie atgalinio URL po registracijos
     * @param string $lang_id Kalbos kodas ISO 639-1 standartu ( naudojamas rodant klausimą )
     * @param string $ident Papildomas registruoto vartotojo identifikatorius ( kai norima viena vartotoja registruoti į prenumeratą daugiau negu vieną karta ) !Dar neveikia!
     * @param string $unregister_reason
     * 
     * @return string URL kur reikia nukreipti vartotoja kad jis būtų išregistruotas iš prenumeratos
     *
     * @see unregister
     */
    function unregisterUrl($options, $lang_id=null, $ident=null, $unregister_reason='') {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $options = $this->prepareParams($options);
        $return = $this->SubscriptionWapUnregister($this->service_id, $lang_id, $options, $ident, $unregister_reason);
        $this->debug( 'Returned: '.var_export($return, true), DEBUG_LEVEL_NORMAL);

        if (!$this->isError() && !empty($return)) {
            return $return;
        }

        return '';

    }

    /**
     * Grąžinama informacija apie vartotoją, kurio id  yra $account_id. 
     * 
     * Gražinam rasto vartotojo informaciją:
     * <code>
     * array(
     *    'service_id' => paslauga (prenumeratos)
     *    'channel_id' => paslaugos kanalas
     *    'account_id' => vartotojo identifikatorius Vero sistemoje
     *    'ident'  => Skirtas registruoti vartotoją į tą pačią paslaugą su skirtingais identifikatoriais
     *    'state' => vartotojo Vero sistemos statusas galimos reikšmės ('open', 'subscribe', 'cancel', 'active',  'active_bill', 'suspended', 'remove', 'remove_op_wait', 'remove_wait')
     *    'status' => vartotojų būsenos ('active' - aktyvus, 'suspended' - neaktyvus, 'removed' - neaktyvus)
     *    'register_date' => registracijos data>
     *    'renew_date' => paskutinio atnaujinimo data
     *    'next_renew_date' => sekančio apmokestinimo data
     *    'valid_till' => vartotojo galiojimo periodas
     *    'unregister_date' => išregistravimo data
     * )
     * </code>
     *
     * @param int $account Vartotojo identifikatorius Vero MBS sistemoje arba MSISDN ( telefono numeris )
     * @param string $account_param Nurodo ar buvo paduotas vartotojo identifikatorisu ( reikšmė 'id' ) ar MSISDN ( reikšmė 'phone' ), pagal nutylėjima paduodamas id
     * @param string $ident Vartotojo papildomas identifikatorius ( naudojamas su paslaugomis, kai galima registruoti vieną vartotoją daugiau negu vieną kartą į prenumeratą )
     *
     * @return array Prenumeruoto vartotojo informacija
     */
    function getInfo($account, $account_param='id', $ident=null) {
        if ( empty($this->user_info[$account][$ident]) ) {
            switch ($account_param) {
                case 'phone':
                $user_info = $this->SubscriptionGetByMSISDN($this->service_id, null, $account, $ident);
                break;

                case 'id':
                default:
                $user_info = $this->SubscriptionGet($this->service_id, null, $account, $ident);
                break;
            }

            if ( !$this->isError() ) {
                $this->user_info[$account][$ident] = $user_info;
            } else {
                return false;
            }
        }
        return $this->user_info[$account][$ident];
    }

    /**
     * Patikrinama ar vartotojas $account_id gali naudotis paslaugomis (Ar yra aktyvus).
     * 
     * @param int $account Vartotojo identifikatorius Vero MBS sistemoje arba MSISDN ( telefono numeris )
     * @param string $account_param Nurodo ar buvo paduotas vartotojo identifikatorisu ( reikšmė 'id' ) ar MSISDN ( reikšmė 'phone' ), pagal nutylėjima paduodamas id
     * @param string $ident Vartotojo papildomas identifikatorius ( naudojamas su paslaugomis, kai galima registruoti vieną vartotoją daugiau negu vieną kartą į prenumeratą )
     *
     * @return boolean
     */
    function isActive($account, $account_param='id', $ident=null) {
        $user_info = $this->getInfo($account, $account_param, $ident);
        return ($user_info['status'] == MBS_SUBSCRIBER_STATUS_ACTIVE);
    }

    /**
     * Grąžinamas vartotojo $account_id statusas prenumeratų sistemoje.
     *
     * Statusai gali būti:
     *
     * # active - {@link MBS_SUBSCRIBER_STATUS_ACTIVE} – vartotojas aktyvus prenumeratorius.
     * # suspend – {@link MBS_SUBSCRIBER_STATUS_SUSPENDED} – vartotojo narystė laikinai sustabdyta, nes nepavyko pratęsti prenumeratos
     * # removed – {@link MBS_SUBSCRIBER_STATUS_REMOVED} – vartotojas nėra prenumeratos narys.
     *
     * @param int $account Vartotojo identifikatorius Vero MBS sistemoje arba MSISDN ( telefono numeris )
     * @param string $account_param Nurodo ar buvo paduotas vartotojo identifikatorisu ( reikšmė 'id' ) ar MSISDN ( reikšmė 'phone' ), pagal nutylėjima paduodamas id
     * @param string $ident Vartotojo papildomas identifikatorius ( naudojamas su paslaugomis, kai galima registruoti vieną vartotoją daugiau negu vieną kartą į prenumeratą )
     *
     * @return string {@link MBS_SUBSCRIBER_STATUS_ACTIVE} | {@link MBS_SUBSCRIBER_STATUS_SUSPENDED} | {@link MBS_SUBSCRIBER_STATUS_REMOVED}
     */
    function getStatus($account, $account_param='id', $ident=null) {
        $user_info = $this->getInfo($account, $account_param, $ident);
        return $user_info['status'];
    }

    /**
     * Gražinamas einamos paslaugos vartotojų sąrašas.
     *
     * Sąrašas - masyvas kurio kiekviename elemente pateikima informacija:
     * <code>
     * array(
     *    'service_id' => paslauga (prenumeratos)
     *    'channel_id' => paslaugos kanalas
     *    'account_id' => vartotojo identifikatorius Vero sistemoje
     *    'ident'  => Skirtas registruoti vartotoją į tą pačią paslaugą su skirtingais identifikatoriais
     *    'state' => vartotojo Vero sistemos statusas galimos reikšmės ('open', 'subscribe', 'cancel', 'active',  'active_bill', 'suspended', 'remove', 'remove_op_wait', 'remove_wait')
     *    'status' => vartotojų būsenos ('active' - aktyvus, 'suspended' - neaktyvus, 'removed' - neaktyvus)
     *    'register_date' => registracijos data>
     *    'renew_date' => paskutinio atnaujinimo data
     *    'next_renew_date' => sekančio apmokestinimo data
     *    'valid_till' => vartotojo galiojimo periodas
     *    'unregister_date' => išregistravimo data
     * )
     * </code>
     *
     * @param int $account_id Vartotojo identifikatorius Vero MBS sistemoje
     *
     * @return array Vartotojo perenumeratų informacija
     */
    function getUserList($account_id) {
        return $this->SubscriptionGetList($account_id, $this->service_id, 0, $this->channel_id);
    }

    /**
     * Paruošiama vartotojo registravimui.
     *
     * Kad vartotojas taptu registruotas reikia jį nukreipti gautu adresu. Gražinami parametrai:
     * transaction_id – sukurtos tranzakcijos id
     * url – adresas kur nukreipti vartotoją, kad įvyktu registracija<br/>
     *
     * @param int $service_id Wap paslaugos identifikatorius
     * @param int $channel_id Kanalo identifikatorius
     * @param string $lang_identi kalbos kodas ,kuria bus rodomi pranešimai ISO 639-1 standartu.
     * @param array $options parametrai kurie bus grąžinami.
     * @param string $ident Skirtas vienam vartotojui būti registruotam vienoje prenumeratoje kelis kartus naudojant skirtingą šį parametrą.
     * @param string $valid_till Nurodo data ir laiką, iki kada galios vartotojo prenumerata
     * 
     * @return array
     */
    function SubscriptionWapRegister( $service_id, $channel_id=null, $lang_identi=null, $options=array(), $ident=null, $valid_till=null ) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        if (empty($valid_till)) {
            $valid_till = null;
        }

        $info = array(
                'wap_service_id' 	=> (int)$service_id,	// Wap service
                'channel_id' 		=> (int)$channel_id,	// Channel ID
                'language_ident' 	=> $lang_identi,		// Language code(ISO 639-1 standard) in which agreement to pay question will be shown
                'options' 			=> $options,			// This is place for send parameters, that should be attached to back URL
                'ident'				=> $ident,				// Extra identifyer to help one user use same subscription several times
                'valid_till'		=> $valid_till,			// Date then user subscription will be terminated (removed)
        );
        $this->initSoap(MBS_SUBSCRIPTION_SOAP_URL);
        $data = $this->_SOAPCall('SubscriptionWapRegister', $info);

        return $data;

    }

    /**
     * Pasiruošimas vartotojo išregistravimui. 
     *
     * Metodas grąžina URL, kuriuo reikia nukreipti vartotoją, kad galutinai jis būtu išregistruotas.
     *
     * @param int $service_id WAP apmokestinimo paslaugos id
     * @param string $lang_ident kalbos kodas ,kuria bus rodomi pranešimai ISO 639-1 standartu.
     * @param array $options parametrai, kurie bus grąžinami.
     * @param string $ident Skirtas vienam vartotojui būti registruotam vienoje prenumeratoje kelis kartus naudojant skirtingą šį parametrą.
     * @param string $unregister_reason Why user is unregistering or identifier of system that unregisters
     *
     * @return string URL
     */
    function SubscriptionWapUnregister($service_id, $lang_ident=null, $options=array(), $ident=null, $unregister_reason=null) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $info = array (
                'wap_service_id' 	=> (int)$service_id,			// Service_id
                'language_ident' 	=> $lang_ident,					// Language ident
                'options' 			=> $options,	// Do nothing no transaction created :(
                'ident'				=> $ident,						// Not implemented. Identifyer to help one user use same subscription
                'unregister_reason' => $unregister_reason,			// Why user is unregistering (or unregistering system name)
        );
        $this->initSoap(MBS_SUBSCRIPTION_SOAP_URL);
        $data = $this->_SOAPCall('SubscriptionWapUnregister', $info);

        return $data;

    }

    /**
     * Registruojamas vartotojas į paslaugą.
     * Metodas grąžina URL, kuriuo reikia nukreipti vartotoją, kad galutinai jis būtu užregistruotas.
     * Jei URL negražino, vartotojas jau užregistruotas.
     *
     * @param int $service_id Prenumeratinės paslaugos identifikatorius
     * @param int $account_id Vartotojo identifikatorius Vero MBS sistemoje
     * @param int $channel_id Kanalo identifikatorius
     * @param string $lang_identi kalbos kodas ,kuria bus rodomi pranešimai ISO 639-1 standartu.
     * @param array $options parametrai kurie bus grąžinami.
     * @param string $ident Skirtas vienam vartotojui būti registruotam vienoje prenumeratoje kelis kartus naudojant skirtingą šį parametrą.
     * @param string $valid_till Nurodo data ir laiką, iki kada galios vartotojo prenumerata
     * 
     * @return string URL
     */
    function SubscriptionRegister($service_id, $account_id, $channel_id=null, $lang_identi=null, $options=array(), $ident=null, $valid_till=null) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $info = array(
                'service_id' 		=> (int)$service_id,		// Subscribe service ID
        		'account_id' 		=> (int)$account_id,	// Account ID
                'channel_id' 		=> (int)$channel_id,	// Channel ID
                'language_ident' 	=> $lang_identi,		// Language code(ISO 639-1 standard) in which agreement to pay question will be shown
                'options' 			=> $options,			// This is place for send parameters, that should be attached to back URL
                'ident'				=> $ident,				// Extra identifyer to help one user use same subscription several times
                'valid_till'		=> $valid_till,			// Date then user subscription will be terminated (removed)
        );
        $this->initSoap(MBS_SUBSCRIPTION_SOAP_URL);
        $data = $this->_SOAPCall('SubscriptionRegister', $info);

        return $data;
    }

    /**
     * Vartotojo išregistravimas.
     * Metodas grąžina URL, kuriuo reikia nukreipti vartotoją, kad galutinai jis būtu išregistruotas.
     * Jei URL negrazino, vartotojas jau išregistruotas.
     *
     * @param int $service_id Prenumeratinės paslaugs identifikatorius
     * @param int $account_id Vartotjo identifikatorius Vero MBS sistemoje
     * @param string $lang_identi kalbos kodas ,kuria bus rodomi pranešimai ISO 639-1 standartu.
     * @param array $options parametrai kurie bus grąžinami.
     * @param string $ident Skirtas vienam vartotojui būti registruotam vienoje prenumeratoje kelis kartus naudojant skirtingą šį parametrą.
     * @param string $unregister_reason Why user is unregistering or identifier of system that unregisters
     */
    function SubscriptionUnregister($service_id, $account_id, $lang_identi=null, $options=array(), $ident=null, $unregister_reason=null) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $info = array (
                'service_id' 		=> (int)$service_id,	// Subscription service
                'account_id' 		=> (int)$account_id,	// User account ID
        		'language_ident'	=> $lang_identi,		// Language code(ISO 639-1 standard) in which agreement to pay question will be shown
                'options' 			=> $options,			// This is place for send parameters, that should be attached to back URL
                'ident'				=> $ident,				// Not implemented. Identifyer to help one user use same subscription
        		'unregister_reason'	=> $unregister_reason,	// Why user is unregistering (or unregistering system name)
        );
        $this->initSoap(MBS_SUBSCRIPTION_SOAP_URL);
        $data = $this->_SOAPCall('SubscriptionUnregister', $info);
        
        return $data;
    }

    /**
     * Gražinama informacija apie vartotoją $account_id.
     *
     * Vienas parametras $wap_service_id arba $service_id turi būti netuščias.
     * # Parametras $wap_service_id – paslaugos, kuri naudojama WAP registracijai/apmokestinimui.
     * # Parametras $service_id –  prenumeratinės paslaugos identifikatorius.
     * # Parametras $ident – vartotojo identifikatorius paslaugoje (jei registruojamas į tą pačią paslaugą kelis kartus).
     *
     * Grąžinamas vartotojo informacija kaip aprašyta {@link getInfo} metode.<br/>
     * 
     * Gražinama struktūra:
     * <code>
     * array(
     *    'service_id' => paslauga (prenumeratos)
     *    'channel_id' => paslaugos kanalas
     *    'account_id' => vartotojo identifikatorius Vero sistemoje
     *    'ident'  => Skirtas registruoti vartotoją į tą pačią paslaugą su skirtingais identifikatoriais
     *    'state' => vartotojo Vero sistemos statusas galimos reikšmės ('open', 'subscribe', 'cancel', 'active',  'active_bill', 'suspended', 'remove', 'remove_op_wait', 'remove_wait')
     *    'status' => vartotojų būsenos ('active' - aktyvus, 'suspended' - neaktyvus, 'removed' - neaktyvus)
     *    'register_date' => registracijos data>
     *    'renew_date' => paskutinio atnaujinimo data
     *    'next_renew_date' => sekančio apmokestinimo data
     *    'valid_till' => vartotojo galiojimo periodas
     *    'unregister_date' => išregistravimo data
     * )
     * </code>
     * 
     *
     * @param int $wap_service_id Wap paslaugos identifikatorius
     * @param int $service_id Prenumeratinės paslaugos identifikatorius
     * @param int $account_id Vartotojo identifikatorius Vero MBS sistemoje
     * @param string $ident Skirtas vienam vartotojui būti registruotam vienoje prenumeratoje kelis kartus naudojant skirtingą šį parametrą.
     *
     * @return array
     *
     */
    function SubscriptionGet($wap_service_id=null, $service_id, $account_id, $ident=null) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $info = array (
                'wap_service_id' 	=> (int)$wap_service_id,	// Wap service. 0 if using subscription service ID
                'service_id' 		=> (int)$service_id,		// subscription ID. 0 if using WAP service ID
                'account_id' 		=> (int)$account_id,		// User account ID
                'ident'				=> $ident,					// Not implemented. Identifyer to help one user use same subscription
        );
        $this->initSoap(MBS_SUBSCRIPTION_SOAP_URL);
        $data = $this->_SOAPCall('SubscriptionGet', $info);

        return $data;
    }

    /**
     * Metodas analogiškas {@link SubscriptionGet}, tik gražina informaciją ne pagal vartotojo identifikatorių o pagal MSISDN ( telefono numerį )
     * Vienas parametras $wap_service_id arba $service_id turi būti netuščias.
     *
     * @param int $wap_service_id Wap paslaugos identifikatorius
     * @param int $service_id Prenumeratinės paslaugos identifikatorius
     * @param int $account_id Vartotojo identifikatorius Vero MBS sistemoje
     * @param string $ident Skirtas vienam vartotojui būti registruotam vienoje prenumeratoje kelis kartus naudojant skirtingą šį parametrą.
     *
     * @return array Vartotojo informacija tokia pati kaip {@link SubscriptionGet} metodo
     *
     * @see SubscriptionGet
     */
    function SubscriptionGetByMSISDN($wap_service_id=null, $service_id, $account_phone, $ident=null) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $info = array (
                'wap_service_id' 	=> (int)$wap_service_id,	// Wap service. 0 if using subscription service ID
                'service_id' 		=> (int)$service_id,		// subscription ID. 0 if using WAP service ID
                'account_phone' 	=> $account_phone,			// User account MSISDN
                'ident'				=> $ident,					// Not implemented. Identifyer to help one user use same subscription
        );
        $this->initSoap(MBS_SUBSCRIPTION_SOAP_URL);
        $data = $this->_SOAPCall('SubscriptionGetByMSISDN', $info);

        return $data;
    }

    /**
     * Gražinami visi paslaugos vartotojai.
     *
     * Vienas parametras $wap_service_id arba $service_id turi būti netuščias.
     * # Parametras $wap_service_id – paslaugos, kuri naudojama WAP registracijai/apmokestinimui.
     * # Parametras $service_id –  prenumeratinės paslaugos identifikatorius.
     *
     * $channel_id – paslaugos kanalas, kuri yra įvesta.
     *
     * Grąžinamas vartotojų sąrašas ( masyvas ) kurio kiekvieno elemento struktūra:
     * <code>
     * array(
     *    'service_id' => paslauga (prenumeratos)
     *    'channel_id' => paslaugos kanalas
     *    'account_id' => vartotojo identifikatorius Vero sistemoje
     *    'ident'  => Skirtas registruoti vartotoją į tą pačią paslaugą su skirtingais identifikatoriais
     *    'state' => vartotojo Vero sistemos statusas galimos reikšmės ('open', 'subscribe', 'cancel', 'active',  'active_bill', 'suspended', 'remove', 'remove_op_wait', 'remove_wait')
     *    'status' => vartotojų būsenos ('active' - aktyvus, 'suspended' - neaktyvus, 'removed' - neaktyvus)
     *    'register_date' => registracijos data>
     *    'renew_date' => paskutinio atnaujinimo data
     *    'next_renew_date' => sekančio apmokestinimo data
     *    'valid_till' => vartotojo galiojimo periodas
     *    'unregister_date' => išregistravimo data
     * )
     * </code>
     *
     * @param int $account_id Vartotojo identifikatorius Vero MBS sistemoje
     * @param int $wap_service_id Wap paslaugos identifikatorius
     * @param int $service_id Prenumeratinės paslaugos identifikatorius
     * @param int $channel_id Kanalo identifikatorius
     * 
     *
     * @return array Vartotojų sąrašas
     *
     */
    function SubscriptionGetList($account_id, $wap_service_id=null, $service_id, $channel_id ) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $info = array (
                'account_id'		=> $account_id,
                'wap_service_id' 	=> $wap_service_id,
                'service_id' 		=> $service_id,
                'channel_id'		=> $channel_id,
        );
        $this->initSoap(MBS_SUBSCRIPTION_SOAP_URL);
        $data = $this->_SOAPCall('SubscriptionGetList', $info);

        return $data;
    }

    /**
     * Atlika tą patį ka ir {@link SubscriptionGetList} metodas, tik vietoj vartotojo identifikatoriaus galima naudoti MSISDN ( telefono numerį )
     * 
     * Vienas parametras $wap_service_id arba $service_id turi būti netuščias.
     *
     * @param int $account_phone Vartotojo telefono numeris
     * @param int $wap_service_id Wap paslaugos identifikatorius
     * @param int $service_id Prenumeratinės paslaugos identifikatorius
     * @param int $channel_id Kanalo identifikatorius
     *
     * @return array Vartotojų sąrašas, detaliau žiūrėkite {@link SubscriptionGetList} metodo aprašymą
     *
     * @see SubscriptionGetList
     */
    function SubscriptionGetListByMSISDN($account_phone, $wap_service_id=null, $service_id, $channel_id ) {
        $this->debug(__FUNCTION__.' are started', DEBUG_LEVEL_HIGH );

        $info = array (
                'account_phone'		=> $account_phone,
                'wap_service_id' 	=> $wap_service_id,
                'service_id' 		=> $service_id,
                'channel_id'		=> $channel_id,
        );
        $this->initSoap(MBS_SUBSCRIPTION_SOAP_URL);
        $data = $this->_SOAPCall('SubscriptionGetListByMSISDN', $info);

        return $data;
    }

}
?>